#!/opt/edge/venv/bin/python3.11

from edge import eventlog
from edge.webserver import cgi
from edge.webserver.sysconf.api_bus_lane import BusLaneApi


class _HandlerFactory:
    """
    GET      /api/v2/sys/conf/bus-lane   /<uiref>
    PUT      /api/v2/sys/conf/bus-lane   /<uiref>
    DELETE   /api/v2/sys/conf/bus-lane   /<uiref>
    """

    @classmethod
    def create(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        method_to_handlers_map: cgi.CgiHandlerFactoryDict = {
            'GET': [
                cls._bus_lane__get_uiref,
            ],
            'PUT': [
                cls._bus_lane__put_uiref
            ],
            'DELETE': [
                cls._bus_lane__delete_uiref
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _bus_lane__get_uiref(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler() -> cgi.CgiResponse:
            uiconf_pub = BusLaneApi.load_detector(request.path_info[0])
            return cgi.CgiResponse(body=uiconf_pub)

        return handler

    @classmethod
    def _bus_lane__put_uiref(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_present()

        def handler() -> cgi.CgiResponse:
            new_uiconf = BusLaneApi.save_detector(request.path_info[0], request.body_jnode)
            return cgi.CgiResponse(body=new_uiconf)

        return handler

    @classmethod
    def _bus_lane__delete_uiref(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler() -> cgi.CgiResponse:
            terms_info = BusLaneApi.drop_detector(request.path_info[0])
            return cgi.CgiResponse(body=terms_info)

        return handler


if __name__ == '__main__':
    eventlog.open_log('cgi-bus-lane', 'debug')
    try:
        cgi.cgi_handle(_HandlerFactory.create)
    finally:
        eventlog.close_log()
