#!/opt/edge/venv/bin/python3.11

from edge import eventlog
from edge.webserver import cgi
from edge.webserver.sysconf.api_origins import OriginsApi


class _HandlerFactory:
    """
    GET   /api/v2/sys/conf/transit
    """

    @classmethod
    def create(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        method_to_handlers_map: cgi.CgiHandlerFactoryDict = {
            'GET': [
                cls._get_origins
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _get_origins(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        if len(request.path_info) != 0:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler() -> cgi.CgiResponse:
            ext_uiconf = OriginsApi.load_unit()
            return cgi.CgiResponse(body=ext_uiconf)

        return handler


if __name__ == '__main__':
    eventlog.open_log('cgi-origins', 'debug')
    try:
        cgi.cgi_handle(_HandlerFactory.create)
    finally:
        eventlog.close_log()
