#!/opt/edge/venv/bin/python3.11

from edge import eventlog
from edge.webserver import cgi
from edge.webserver.sysconf.api_spot_speed import SpotSpeedWagaApi


class _HandlerFactory:
    """
    GET      /api/v2/sys/conf/spot-speed-waga   /<uiref>
    PUT      /api/v2/sys/conf/spot-speed-waga   /<uiref>
    DELETE   /api/v2/sys/conf/spot-speed-waga   /<uiref>
    """

    @classmethod
    def create(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        method_to_handlers_map: cgi.CgiHandlerFactoryDict = {
            'GET': [
                cls._spot_speed__get_uiref,
            ],
            'PUT': [
                cls._spot_speed__put_uiref
            ],
            'DELETE': [
                cls._spot_speed__delete_uiref
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _spot_speed__get_uiref(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        request.assert_query_absent()
        request.assert_body_absent()

        def handler() -> cgi.CgiResponse:
            uiconf = SpotSpeedWagaApi.load_detector(request.path_info[0])
            return cgi.CgiResponse(body=uiconf)

        return handler

    @classmethod
    def _spot_speed__put_uiref(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        request.assert_query_absent()
        request.assert_body_present()

        def handler() -> cgi.CgiResponse:
            new_uiconf = SpotSpeedWagaApi.save_detector(request.path_info[0], request.body_jnode)
            return cgi.CgiResponse(body=new_uiconf)

        return handler

    @classmethod
    def _spot_speed__delete_uiref(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        request.assert_query_absent()
        request.assert_body_absent()

        def handler() -> cgi.CgiResponse:
            terms_info = SpotSpeedWagaApi.drop_detector(request.path_info[0])
            return cgi.CgiResponse(body=terms_info)

        return handler


if __name__ == '__main__':
    eventlog.open_log('cgi-spot-speed-waga', 'debug')
    try:
        cgi.cgi_handle(_HandlerFactory.create)
    finally:
        eventlog.close_log()
