#!/opt/edge/venv/bin/python3.11

from edge import eventlog
from edge.webserver import cgi
from edge.webserver.sysconf.api_users import UsersApi


class _HandlerFactory:
    """
    GET      /api/v2/sys/conf/users   /<uiref>
    PUT      /api/v2/sys/conf/users   /<uiref>/<login>
    """

    @classmethod
    def create(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        method_to_handlers_map: cgi.CgiHandlerFactoryDict = {
            'GET': [
                cls._users__get_uiref,
            ],
            'PUT': [
                cls._users__put_uiref_login
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _users__get_uiref(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler() -> cgi.CgiResponse:
            unit_ext_jnode = UsersApi.load_unit(request.path_info[0])
            return cgi.CgiResponse(body=unit_ext_jnode)

        return handler

    @classmethod
    def _users__put_uiref_login(cls, request: cgi.CgiRequest) -> cgi.CgiHandler:
        if len(request.path_info) != 2:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_present()

        def handler() -> cgi.CgiResponse:
            unit_uiref, user_login = request.path_info
            UsersApi.edit_user(unit_uiref, user_login, request.body_jnode)
            return cgi.CgiResponse()

        return handler


if __name__ == '__main__':
    eventlog.open_log('cgi-users', 'debug')
    try:
        cgi.cgi_handle(_HandlerFactory.create)
    finally:
        eventlog.close_log()
