from PIL import ImageFont
from typing import Final

from edge.image.helpers import ImHelpers
from edge.image.profiles import FONT_PATH2
from edge.types.blob import MemBlob
from edge.image.types.footer import ImFooterVars
from edge.types.image import ImRgba
from edge.image.types.types import LiveImConf


class LiveImProfile:
    _FONT_PT_SIZE = 20

    def __init__(self, args: LiveImConf):
        self._conf: Final[LiveImConf] = args
        self._font = ImageFont.truetype(FONT_PATH2, args.footer.font_size, encoding='utf-8')

    def process(self, scene_dh_data: ImRgba, footer_vars: ImFooterVars) -> MemBlob:
        # render footer
        footer_conf = self._conf.footer.eval(footer_vars)
        scene_dh_data.with_footer_image(
            lambda im: ImHelpers.draw_footer(im, footer_conf, self._font, self._conf.footer.font_size))

        # render annotations
        blob_data = scene_dh_data.with_image(lambda im: ImHelpers.mk_jpeg(im))

        return MemBlob('live', scene_dh_data.ts, blob_data)
