import datetime

from typing import Dict,  Final, Optional

from edge.image.types.footer import ImFooterConf


class PlateImConf:
    def __init__(self):
        self.width, self.height = 134, 30  # ~ 5:1


class ThumbImConf:
    def __init__(self):
        # self.width, self.height = 220, 165  # 4:3
        self.width, self.height = 224, 224  # 1:1 for MMR


class VehicleImAnnoConf:
    def __init__(self, plate: bool = False):
        self.plate: Final[bool] = plate


class VehicleImConf:
    def __init__(self, width: int, footer: ImFooterConf, anno: VehicleImAnnoConf):
        self.wh_ratio: Final[float] = 4.0 / 3.0

        self.width: Final[int | float] = width
        self.height: Final[int | float] = width / self.wh_ratio  # 4:3

        self.footer: Final[ImFooterConf] = footer
        self.anno: Final[VehicleImAnnoConf] = anno


class SceneImAnnoConf:
    def __init__(self, track: bool = False, vehicle: bool = False, plate: bool = False):
        self.track: Final[bool] = track
        self.vehicle: Final[bool] = vehicle
        self.plate: Final[bool] = plate


class SceneImConf:
    def __init__(self, width: int, footer: ImFooterConf, anno: SceneImAnnoConf):
        self.width: Final[int] = width
        self.footer: Final[ImFooterConf] = footer
        self.anno: Final[SceneImAnnoConf] = anno


class LiveImConf:
    def __init__(self, footer: ImFooterConf):
        self.footer: Final[ImFooterConf] = footer


class ContrastScheduleConf:
    MODE_ALWAYS = 'always'
    MODE_NEVER = 'never'
    MODE_NIGHT = 'twilight'
    MODE_TIME = 'time'

    def __init__(self, mode: str,
                 time_from: Optional[datetime.time] = None,
                 time_to: Optional[datetime.time] = None):

        self.mode: Final[str] = mode
        self.time_from: Final[datetime.time | None] = time_from
        self.time_to: Final[datetime.time | None] = time_to


class ContrastClaheConf:
    def __init__(self, clip_limit: float = 10.0, grid_cols: int = 8, grid_rows: int = 8):
        self.clip_limit: Final[float] = clip_limit
        self.grid_cols: Final[int] = grid_cols
        self.grid_rows: Final[int] = grid_rows

    def __eq__(self, that: object) -> bool:
        if not isinstance(that, ContrastClaheConf):
            return False

        return self.clip_limit == that.clip_limit and \
            self.grid_cols == that.grid_cols and \
            self.grid_rows == that.grid_rows


class ContrastConf:
    def __init__(self,
                 shedule: ContrastScheduleConf,
                 clache: ContrastClaheConf):

        self.shedule: Final[ContrastScheduleConf] = shedule
        self.clache: Final[ContrastClaheConf] = clache


class EventMeasImBuilderConf:
    def __init__(self,
                 scene_im: SceneImConf,
                 scene_footer: Dict,
                 scene_contrast: ContrastConf,
                 vehicle_im: Optional[VehicleImConf],
                 vehicle_footer: Optional[Dict],
                 vehicle_contrast: ContrastConf):

        self.plate: Final[PlateImConf] = PlateImConf()
        self.thumb: Final[ThumbImConf] = ThumbImConf()

        self.vehicle: Final[VehicleImConf] = vehicle_im
        self.vehicle_footer: Final[Dict] = vehicle_footer
        self.vehicle_contrast = vehicle_contrast

        self.scene: Final[SceneImConf] = scene_im
        self.scene_footer: Final[Dict] = scene_footer
        self.scene_contrast = scene_contrast
