from typing import Final, List, Optional, Tuple

from edge.services.types.service import IService
from edge.types.track import LrTrackItem, TrTrackItem


class CamServiceConf:
    def __init__(self, intrinsics_path: str, extrinsics_path: str, avg_plate_u: Optional[float], with_log: bool):
        self.intrinsics_path: Final[str] = intrinsics_path
        self.extrinsics_path: Final[str] = extrinsics_path
        self.avg_plate_u: Final[float | None] = avg_plate_u
        self.with_log: Final[bool] = with_log


class ICamService(IService):
    def mk_image_point(self, scene_l: float, scene_u: float, scene_f: float) -> Tuple[int, int]:
        """
        :raises ParamsError:
        """
        raise NotImplementedError()

    def mk_world_point(self, image_x: int, image_y: int, world_t: float) -> Tuple[float, float]:
        """
        :raises ParamsError:
        """
        raise NotImplementedError()

    def assert_racam_speed_2(self,
                             vrp_text_utf8: bytes,
                             main_speed_kmh: int,
                             items: List[TrTrackItem],
                             slow_vehicle: bool) -> int:
        """
        :returns:
            Positive value means that the vehicle is moving in receeding direction relative to the control zone
            Negative value means that the vehicle is moving in approaching direction relative to the control zone

        :raises ParamsError:
            Failed to estimate or validate the speed.
            The reason is invalid input parameters.
        """
        raise NotImplementedError()

    def assert_cam_speed_2(self,
                           vrp_text_utf8: bytes,
                           main_speed_kmh: int,
                           min_lr_item: LrTrackItem,
                           max_lr_item: LrTrackItem,
                           mmr_class: str,
                           main_dir_u: bool = False) -> int:
        """
        :returns:
            Positive value means that the vehicle is moving in receeding direction relative to the control zone
            Negative value means that the vehicle is moving in approaching direction relative to the control zone

        :raises ParamsError:
            Failed to estimate or validate the speed.
            The reason is invalid input parameters.
        """
        raise NotImplementedError()
