from typing import Optional, Tuple

from edge.services.types.service import IService
from edge.types.geoposition import Geoposition


class GnssServiceConf:
    PROVIDER_STATIC = 'static'
    PROVIDER_GPSD = 'gpsd'
    _MAX_ENTRY_COUNT = 100
    _MAX_AGE_DELTA_USEC = int(60 * 1e6)
    _MAX_LOOKUP_DIST_USEC = int(5 * 1e6)

    def __init__(self, provider: str = PROVIDER_STATIC,
                 geodata: Optional[Geoposition] = None,
                 endpoint: Optional[Tuple[str, int]] = None,
                 azimuth: Optional[float] = None,
                 max_entry_count: int = _MAX_ENTRY_COUNT,
                 max_age_delta_usec: int = _MAX_AGE_DELTA_USEC,
                 max_lookup_dist_usec: int = _MAX_LOOKUP_DIST_USEC):

        self.provider = provider
        self.azimuth = azimuth
        self.geodata = geodata
        self.endpoint = endpoint
        self.max_entry_count = max_entry_count
        self.max_age_delta_usec = max_age_delta_usec
        self.max_lookup_dist_usec = max_lookup_dist_usec


class IGnssService(IService):
    def __init__(self):
        super().__init__()

    def get_geoposition(self, ts: int) -> Optional[Geoposition]:
        raise NotImplementedError
