from edge.types.exceptions import ParamsError


def assert_speed_rel_error(main_speed: int, control_speed: int, max_rel_error: float) -> bool:
    """
    :param main_speed:
        Positive value means that the vehicle is moving in receeding direction relative to the control zone
        Negative value means that the vehicle is moving in approaching direction relative to the control zone
    :param control_speed:
        Positive value means that the vehicle is moving in receeding direction relative to the control zone
        Negative value means that the vehicle is moving in approaching direction relative to the control zone
    :param max_rel_error:
        Value is always positive.
    """
    rel_error: float = (main_speed - control_speed) / main_speed
    return abs(rel_error) <= max_rel_error


def assert_speed_rel_error2(main_speed: int, control_speed: int, max_rel_error: float) -> bool:
    """
    :param main_speed:
        Value is always positive. The direction of vehicle movement is unknown.
        It is assumed that the direction of the main speed is the same as the direction of the control speed.
    :param control_speed:
        Positive value means that the vehicle is moving in receeding direction relative to the control zone
        Negative value means that the vehicle is moving in approaching direction relative to the control zone
    :param max_rel_error:
        Value is always positive.
    """
    rel_error: float = (main_speed - abs(control_speed)) / main_speed
    return abs(rel_error) <= max_rel_error


class SlowVehicleError(ParamsError):
    pass


class FastVehicleError(ParamsError):
    pass


def assert_speed(main_speed_kmh: int, control_speed_kmh: int, mmr_class: str, main_dir_u: bool = False) -> int:
    # accept tracks with cam-speed difference less than 10% of main speed
    if main_dir_u:
        if assert_speed_rel_error2(main_speed_kmh, control_speed_kmh, 0.10):
            return control_speed_kmh
    else:
        if assert_speed_rel_error(main_speed_kmh, control_speed_kmh, 0.10):
            return control_speed_kmh

    # reject tracks with vehicle class is UNKNOWN, BUS, LT, HT
    if mmr_class in ['UNKNOWN', 'BUS', 'LT', 'HT']:
        raise SlowVehicleError()

    # reject tracks with main speed more than 90 km/h
    if main_speed_kmh >= 90:
        raise FastVehicleError()

    return control_speed_kmh


def map_speed_int(speed: float) -> int:
    return int(round(speed))


def map_speed_uint(speed: int | None) -> int | None:
    return speed and abs(speed)
