from string import Template
from typing import Set


class FormatTemplate(Template):
    idpattern = r'[_a-z][-_a-z0-9]*'

    _KNOWN_PHOLDERS: Set[str] = {
        'datetime',
        'datetime-ms',
        'datetime-control',
        'datetime-control-ms',
        'time-control-ms',
        'post-name',
        'geolocation',
        'speed',
        'plate',
        'serial',
        'lane',
        'dir',
        'video-speed',
        'video-speed2',
        'radar-speed',
        'radar-speed2',
        'max-speed',
        'speed-limit',
        'radar-overspeed',
        'model',
        'vendor',
        'section-speed',
        'section-overspeed',
        'wim-speed',
        'main-speed',
        'main-speed2',
        'control-speed',
        'main-overspeed',
        'wim-overspeed',
        'halt-time'
    }

    # noinspection PyUnresolvedReferences
    @classmethod
    def get_pholders(cls, string_format: str) -> Set[str]:
        """
        :raise: ValueError
        """
        template = cls(string_format)

        if not template.is_valid():
            raise ValueError

        string_set = set(template.get_identifiers())
        if not string_set.issubset(FormatTemplate._KNOWN_PHOLDERS):
            raise ValueError

        return string_set
