from typing import Final, Optional, Self


class Geoposition:
    def __init__(self,
                 latitude: float | int,
                 longitude: float | int,
                 altitude: Optional[float] = None,
                 speed: Optional[float] = None,
                 azimuth: Optional[float] = None):
        """
        :param latitude:
            Latitude in degrees: +/- signifies North/South
        :param longitude:
            Longitude in degrees: +/- signifies East/West
        :param altitude:
            Altitude in meters
        :param speed:
            Speed over ground, meters per second
        :param azimuth:
        """

        # http://catb.org/gpsd/gpsd_json.html
        self.latitude: Final[float | int] = latitude
        self.longitude: Final[float | int] = longitude
        self.altitude: Final[float | None] = altitude
        self.speed: Final[float | None] = speed
        self.azimuth: float | None = azimuth  # TODO: make it final

    def __str__(self) -> str:
        # <latChar><DDD.uuuuuu> <longChar><ddd.uuuuuu>
        geodata_str = '{}{:.6f} {}{:.6f}'.format(
            'N' if self.latitude >= 0 else 'S', abs(self.latitude),
            'E' if self.longitude >= 0 else 'W', abs(self.longitude))
        return geodata_str

    def __eq__(self, other: Self) -> bool:
        if not isinstance(other, self.__class__):
            return False

        dlat = abs(self.latitude - other.latitude)
        dlong = abs(self.longitude - other.longitude)

        return dlat < 0.01 and dlong < 0.01

    def __ne__(self, other) -> bool:
        return not self.__eq__(other)
