from typing import Tuple, List, Dict

from edge.types.timespan import decode__timespan
from edge.types.trend import ImPlace, LrTrendItem, LrTrend


def encode__im_place(self: ImPlace) -> Tuple[int, int, int]:
    return self._r, self._min_x + self._r, self._min_y + self._r


def decode__im_place(jnode: List[int]) -> ImPlace:
    if len(jnode) != 3:
        raise ValueError

    return ImPlace(*jnode)


def encode__lr_trend_item(self: LrTrendItem) -> Dict:
    jnode = {
        'place': encode__im_place(self.place),
        'range': [self.range.min_ts, self.range.max_ts]
    }

    if self.event:
        jnode['event'] = True

    return jnode


def decode__lr_trend_item(jnode: Dict) -> LrTrendItem:
    place = decode__im_place(jnode['place'])
    span = decode__timespan(jnode['range'])
    return LrTrendItem(place, span, bool(jnode.get('event')))


def encode__lr_trend(self: LrTrend) -> Dict:
    return {
        'plate-wh': [self.max_vrp_bbox_w, self.max_vrp_bbox_h],
        'timeline': [encode__lr_trend_item(item) for item in self]
    }


def decode__lr_trend(jnode: Dict) -> LrTrend:
    max_plate_wh = jnode['plate-wh']
    items: List[LrTrendItem] = [decode__lr_trend_item(x) for x in jnode['timeline']]
    return LrTrend(max_plate_wh, items)
