import fcntl
import os

from typing import List, Set, Union

from edge import eventlog
from edge.types.exceptions import *
from edge.webserver.sysconf import ph


class UimodsLocker:
    def __init__(self, uimods: Union[Set[ph.UiMod], Set[ph.UiMod2]]):
        self._fd_list: List[int] = []
        try:
            self._fd_list = [self._lock(uimod) for uimod in uimods]
        except Exception:
            self.release()
            raise

    @staticmethod
    def _lock(uimod: Union[ph.UiMod, ph.UiMod2]) -> int:
        dir_path = os.path.dirname(uimod.lock_path)
        if not os.path.dirname(dir_path):
            os.makedirs(dir_path)

        lock_fd = os.open(uimod.lock_path, os.O_CREAT | os.O_TRUNC | os.O_WRONLY)
        try:
            # Request exclusive (EX) non-blocking (NB) advisory lock.
            fcntl.lockf(lock_fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
            return lock_fd

        except IOError as x:
            os.close(lock_fd)
            eventlog.log_d(f'fail: {x}')
            raise BusyError

    @staticmethod
    def _unlock(lock_fd: int) -> None:
        # Request exclusive (EX) non-blocking (NB) advisory lock.
        fcntl.lockf(lock_fd, fcntl.LOCK_UN)
        os.close(lock_fd)

    def release(self) -> None:
        # noinspection PyBroadException
        try:
            list(map(lambda lock_fd: self._unlock(lock_fd), self._fd_list))
        except Exception:
            eventlog.log_x()
            os.abort()
