from typing import Dict, Optional, Self

from edge.support import fs


class BsDgconfEditor65535:
    _DGCONF_PATH = '/etc/edge/blobstore.65535/main.conf'

    def __init__(self, jnode: Dict):
        self._jnode = jnode

    @property
    def jnode(self) -> Dict:
        return self._jnode

    @classmethod
    def load(cls, **kwargs) -> Optional[Self]:
        conf_jnode = fs.load_json3(cls._DGCONF_PATH, **kwargs)
        return cls(conf_jnode) if conf_jnode else None

    def save(self) -> None:
        fs.write_json_file(self.jnode, self._DGCONF_PATH, pretty=True)

    @property
    def chan_count(self) -> int:
        return len(self.jnode['storage'])

    @property
    def metrics_path(self) -> str:
        return self.jnode['monitoring']['stat-path']

    def append_chan(self, chanref: str, es_uiconf: Dict) -> None:
        cleanup_uiconf = es_uiconf['rw']['args']['cleanup']

        archive_limits = dict()

        if cleanup_uiconf['max-du'][0]:
            archive_limits['max-du'] = cleanup_uiconf['max-du'][1]

        if cleanup_uiconf['min-df'][0]:
            archive_limits['min-df'] = cleanup_uiconf['min-df'][1]

        if cleanup_uiconf['max-abs-time'][0]:
            archive_limits['max-abs-time'] = cleanup_uiconf['max-abs-time'][1] * 60

        if cleanup_uiconf['max-rel-time'][0]:
            archive_limits['max-rel-time'] = cleanup_uiconf['max-rel-time'][1] * 60

        self.jnode['storage'].append({
            'channel': chanref,
            'cache': {
                'directory': '/var/opt/edge/blobstore/65535-cache',
                'limits': {
                    'max-du': 50,
                    'max-abs-time': 1 * 60
                }
            },
            'archive': {
                'directory': '/var/opt/edge/blobstore/65535',
                'filesize': 4 * 1024,
                'limits': archive_limits
            }
        })

    def remove_chan(self, chanref: str) -> None:
        self.jnode['storage'] = [x for x in self.jnode['storage'] if x['channel'] != chanref]
