from typing import Dict, List, Tuple

from edge.types.identity import Identity


def create_unit__media_server__media(unit_name: str, react_dir: str) -> Tuple[str, Dict]:
    uiref = Identity(unit_name)
    parent_uiref = uiref.parent

    return unit_name, {
        'bundle': ['libunity_edge', '1.0.0'],
        'type': parent_uiref.type,
        'name': parent_uiref.ref,
        'units':
        [
            {
                'type': uiref.type,
                'name': uiref.ref,
                'conf': {
                    'endpoint': react_dir
                }
            }
        ]
    }


def create_unit__event_emitter__tx_track_bus(
        unit_name: str, translate_plate: bool, blob_endpoint: str, args_endpoint: str) -> Tuple[str, Dict]:

    uiref = Identity(unit_name)
    parent_uiref = uiref.parent

    return unit_name, {
        'bundle': ['libunity_edge', '1.0.0'],
        'type': parent_uiref.type,
        'name': parent_uiref.ref,
        'units':
        [
            {
                'type': uiref.type,
                'name': uiref.ref,
                'conf': {
                    'translate-plate': translate_plate,
                    'store': {
                        'endpoint': blob_endpoint,
                        'min-file-count': 20,
                        'max-file-count': 25
                    },
                    'track': [
                        args_endpoint
                    ]
                }
            }
        ]
    }


def create_unit__event_emitter__tx_scene_bus(unit_name: str, endpoints: List[str]) -> Tuple[str, Dict]:
    uiref = Identity(unit_name)
    parent_uiref = uiref.parent

    return unit_name, {
        'bundle': ['libunity_edge', '1.0.0'],
        'type': parent_uiref.type,
        'name': parent_uiref.ref,
        'units':
        [
            {
                'type': uiref.type,
                'name': uiref.ref,
                'conf': {
                    'scene': endpoints
                }
            }
        ]
    }
