from typing import Dict, Tuple

from edge.types.identity import Identity


def create_unit__rule___rule(unit_name: str, uiconf: Dict) -> Tuple[str, Dict]:
    uiref = Identity(unit_name)
    parent_uiref = uiref.parent

    pub = uiconf['rw']
    pub_conf = pub['conf']
    action0 = pub_conf['actions'][0]

    return unit_name, {
        'bundle': ['libunity_recipient', '1.0.0'],
        'type': parent_uiref.type,
        'name': parent_uiref.ref,
        'units':
        [
            {
                'type': uiref.type,
                'name': uiref.ref,
                'conf': {
                    'trigger': action0['trigger']
                }
            }
        ]
    }


def create_unit__rule___rule2(unit_name: str, trigger: str) -> Tuple[str, Dict]:
    uiref = Identity(unit_name)
    parent_uiref = uiref.parent

    return unit_name, {
        'bundle': ['libunity_recipient', '1.0.0'],
        'type': parent_uiref.type,
        'name': parent_uiref.ref,
        'units':
        [
            {
                'type': uiref.type,
                'name': uiref.ref,
                'conf': {
                    'trigger': trigger
                }
            }
        ]
    }


def create_unit__recipient___vast2(
        unit_name: str,
        uiconf: Dict,
        dir_a: str | None,
        dir_r: str | None,
        dir_u: str | None,
        term_uuid: str | None) -> Tuple[str, Dict]:
    """
    :param unit_name
        tcp-recipient.0_0
    :param uiconf
    :param dir_a:
    :param dir_r:
    :param dir_u:
    :param term_uuid:
    """
    pub = uiconf['rw']
    pub_conf = pub['conf']

    action0 = pub_conf['actions'][0]
    media0 = action0['media']

    return unit_name, {
        'bundle': ['libunity_recipient', '1.0.0'],
        'type': 'vast2',
        'name': unit_name,
        'units':
        [
            {
                'type': 'vast2~recipient',
                'name': unit_name,
                'conf': {
                    'endpoint': {
                        'host': pub_conf['server']['host'],
                        'port': pub_conf['server']['port'],
                    },
                    'package': {
                        'plate': media0['plate'],
                        'vehicle': media0['vehicle'],
                        'remark': media0['remark'],
                        'confidence': media0.get('confidence', True),
                        'country': media0.get('country', True),
                        'country-a2': media0.get('country-a2', False),
                        'data source': media0.get('data source', ''),
                        'geolocation': media0.get('geolocation', True),
                        'lane': media0.get('lane', True),
                        'mac': media0.get('mac', True),
                        'text': media0.get('text', True),
                        'text-trans': media0.get('text-trans', True),
                        'timestamp2': media0.get('timestamp2', True),
                        'origin': term_uuid,
                        'dir-alias': {
                            'unknown': dir_u,
                            'approaching': dir_a,
                            'receding': dir_r
                        }
                    }
                }
            }
        ]
    }
