from typing import List, Optional, Self, Set, Union

from edge.webserver.sysconf import ph
from edge.webserver.sysconf.dgmod import dg
from edge.webserver.sysconf.dgmod.creator import BaseCreator, UiModToConfMap


# noinspection PyUnusedLocal
def lbs_get_creators(entry_uimod: Union[ph.UiMod, ph.UiMod2],
                     all_used_uimods: Set[Union[ph.UiMod, ph.UiMod2]],
                     uimod_to_newconf_map: UiModToConfMap,
                     forced: bool = False) -> List[BaseCreator]:

    if entry_uimod.type != ph.UI_TYPE_MEDIASTORE:
        return []

    try:
        ms_uimod = next(filter(lambda x: x.type == ph.UI_TYPE_MEDIASTORE, all_used_uimods))
        if forced:
            return [LbsCreator(ms_uimod)]

    except StopIteration:
        pass
    return []


class LbsCreator(BaseCreator):
    def __init__(self, ms_uimod: ph.UiMod):
        BaseCreator.__init__(self)
        self._ms_uimod = ms_uimod
        self._lbs_dgmod = dg.DgMod.create(dg.EDGE_TYPE_LBSAPP, ms_uimod.pid, ms_uimod.id)
        self._ss = None

    @property
    def dgmod(self) -> dg.DgMod:
        return self._lbs_dgmod

    def prepare(self, uimod_to_newconf_map: UiModToConfMap) -> None:
        pass

    def complete(self) -> None:
        edge_lbs = self._lbs_dgmod
        edge_lbs.install()
        edge_lbs.save_setupstore_conf(active=True)

    def rollback(self) -> None:
        pass

    def notify(self) -> None:
        dg.LifeCycle.restart_dgmod(self.dgmod.type, self.dgmod.id)


class LbsDeleter:
    def __init__(self, dgmod: dg.DgMod):
        self._dgmod = dgmod

    @classmethod
    def create(cls, entry_uimod: ph.UiMod) -> Optional[Self]:
        if entry_uimod.type == ph.UI_TYPE_MEDIASTORE:
            dgmod = dg.DgMod.create(dg.EDGE_TYPE_LBSAPP, entry_uimod.pid, entry_uimod.id)
            return cls(dgmod)

    def run(self) -> None:
        self._dgmod.uninstall()
