from typing import Dict, List

from edge.support import fs


class MainConfBuilder:
    def __init__(self) -> None:
        self._jnode = {
            'Analyzer': {
                'ParamSet': {
                    'Target': {
                        'Median': [70, 150],
                        'SteadyState': {
                            'Unstable': 5,
                            'PreStable': 15,
                            'Stable': 30
                        }
                    },
                    'Sweep': {
                        'Gamma': [0.4, 1.5, 0.1],
                        'Lut12': {
                            'Default': '/opt/edge/share/setupstore/edge/mediagrabber/lut12.default.conf',
                            'Evening': '',
                            'Night': '/opt/edge/share/setupstore/edge/mediagrabber/lut12.olvia.conf',
                        }
                    }
                }
            },
            'FileApi': {
                'AddSceneImgReact': {},
                'AddSceneAoiReact': {},
                'SceneControlEvent': []
            }
        }

    def setup_ir_control(self, control_ir: bool) -> None:
        self._jnode['ControlIR'] = control_ir

    def setup_gain_conf(self, gain_conf: List[float]) -> None:
        self._jnode['Analyzer']['ParamSet']['Sweep']['Gain'] = gain_conf

    def setup_exp_time_conf(self, exp_time_conf: List[int]) -> None:
        self._jnode['Analyzer']['ParamSet']['Sweep']['ExpTime'] = exp_time_conf

    def setup_add_scene_img_react(self, spool_dir: str, react_dir: str) -> None:
        self._jnode['FileApi']['AddSceneImgReact']['Url'] = fs.join_path(spool_dir, react_dir)

    def setup_add_scene_aoi_react(self, spool_dir: str, react_dir: str) -> None:
        self._jnode['FileApi']['AddSceneAoiReact']['Url'] = fs.join_path(spool_dir, react_dir)

    def setup_add_scene_control_event(self, event_dir: str) -> None:
        self._jnode['FileApi']['SceneControlEvent'].append(event_dir)

    def build(self) -> Dict:
        return self._jnode
