from typing import Dict, Final

from edge.webserver.sysconf.dgmod.mg.mg_halt import HaltConfBuilder
from edge.webserver.sysconf.dgmod.mg.mg_main import MainConfBuilder
from edge.webserver.sysconf.dgmod.mg.mg_misc import MiscConfBuilder
from edge.webserver.sysconf.dgmod.mg.uiconf import IUiConfProvider


class MvCameraUiConfProvider(IUiConfProvider):
    def __init__(self, uimod_id: str, uiconf: Dict) -> None:
        self._uimod_id: Final[str] = uimod_id
        self._uiconf: Final[Dict] = uiconf

    def setup_builders(self,
                       main_builder: MainConfBuilder,
                       misc_builder: MiscConfBuilder,
                       halt_builder: HaltConfBuilder) -> None:

        pri_conf: Dict = self._uiconf['ro']
        pub_conf: Dict = self._uiconf['rw']

        vs_jnode = pub_conf['videostream']
        vs_control_jnode = vs_jnode['control']
        vs_sensor_jnode = vs_jnode['resolution']
        vs_sensor_fps = vs_jnode['fps']

        raw_format = pri_conf['raw-format']

        main_builder.setup_scene_event_blobs(
            'mg{0}-mg{0}-store-raw'.format(main_builder.uimod_id),
            vs_sensor_fps * 2, vs_sensor_fps * 2 + 5, raw_format)

        main_builder.setup_cam_model(pub_conf['vendor'], pub_conf['model'], pub_conf['serial'])
        main_builder.setup_cam_auqisition(
            vs_sensor_jnode['w'], vs_sensor_jnode['h'],
            vs_sensor_jnode['x'], vs_sensor_jnode['y'],
            vs_sensor_fps, vs_jnode['rotate180'])

        if vs_control_jnode['profile'] == 'manual':
            main_builder.setup_cam_sweep('Manual')
            main_builder.setup_cam_sweep_manual(
                vs_control_jnode['exposure-manual'],
                vs_control_jnode['gain-manual'])
            main_builder.setup_cam_sweep_auto([0, 0], [0, 0])

        else:
            main_builder.setup_cam_sweep('Auto')
            main_builder.setup_cam_sweep_manual(0, 0)
            main_builder.setup_cam_sweep_auto(
                [vs_control_jnode['exposure-auto-min'], vs_control_jnode['exposure-auto-max']],
                [vs_control_jnode['gain-auto-min'], vs_control_jnode['gain-auto-max']])

        ir_jnode = pri_conf.get('ir')
        if ir_jnode is not None:
            main_builder.setup_cam_ir(
                ir_jnode.get('source', 'ExposureActive'),
                ir_jnode['pin'],
                ir_jnode['sync-inverter'])

        main_builder.setup_cam_misc(pri_conf['transfer-data-size'])

        if raw_format == 'MONO8T':
            main_builder.setup_scene_event_args(['mg{0}-mcgst{0}-scene'.format(main_builder.uimod_id)])

            if vs_control_jnode['profile'] == 'auto-70-150':
                main_builder.setup_scene_event_args(['mg{0}-gc{0}-scene'.format(main_builder.uimod_id)])

        else:
            main_builder.setup_scene_event_args(['mg{0}-rc{0}-scene'.format(main_builder.uimod_id)])

        misc_jnode = pri_conf['misc']
        if misc_jnode.get('TsAdjNs') is not None:
            misc_jnode['TsAdjNs'] = 0  # TODO: why it forces 0?

        misc_builder.setup(misc_jnode)

        halt_lines = pri_conf.get('haltaction')
        if halt_lines:
            halt_builder.setup_lines(halt_lines)
