from typing import Dict, Final, List, Optional

from edge.support import fs


class MainConfBuilder:
    def __init__(self, uimod_id: str, spool_dir: str) -> None:
        self.uimod_id: Final[str] = uimod_id
        self._spool_dir: Final[str] = spool_dir
        self._jnode = {
            'api-server': {
                'endpoint': fs.join_path(spool_dir, f'ra{uimod_id}-ra{uimod_id}-uds')
            },
            'monitoring': {
                'stat-file': fs.join_path(spool_dir, 'zabbix', f'ra{uimod_id}'),
                'stat-tout': 30
            },
            'scene-event': {
                'endpoints': list()
            }
        }

    def setup_device(self, driver_name: str, endpoint: str) -> None:
        self._jnode['device'] = {
            'driver': f'/opt/edge/libexec/radar/{driver_name}.so',
            'endpoint': endpoint,
            'runtime': f'/etc/edge/radar.{self.uimod_id}/runtime/'
        }

    def setup_strobe_react(self, fifo_name: str, offset_us: int | float) -> None:
        self._jnode['strobe-react'] = {
            'endpoint': fs.join_path(self._spool_dir, fifo_name),
            'offset': int(offset_us)
        }

    def setup_scene_event_zone(self, conf: Optional[Dict]) -> None:
        """
        :param conf:
            location#/ro/ra-zone
        """
        if conf is not None:
            self._jnode['scene-event'].update({
                'zone': conf
            })

    def setup_scene_event_args(self, args_dirs: List[str]) -> None:
        dirs_jnode: List[str] = self._jnode['scene-event']['endpoints']

        for args_dir in args_dirs:
            dirs_jnode.append(fs.join_path(self._spool_dir, args_dir))

    def build(self) -> Dict:
        return self._jnode
