from typing import Dict, Final, List

from edge.webserver.sysconf.dgmod.ra.ra_api import ApiConfBuilder
from edge.webserver.sysconf.dgmod.ra.ra_halt import HaltConfBuilder
from edge.webserver.sysconf.dgmod.ra.ra_main import MainConfBuilder
from edge.webserver.sysconf.dgmod.ra.uiconf import IUiConfProvider


class LocationUiConfProvider(IUiConfProvider):
    def __init__(self, uimod_id: str, uiconf: Dict, uiconf_changed: bool) -> None:
        self._uimod_id: Final[str] = uimod_id
        self._uiconf: Final[Dict] = uiconf
        self._uiconf_changed: Final[bool] = uiconf_changed

    def setup_builders(self,
                       api_builder: ApiConfBuilder | None,
                       main_builder: MainConfBuilder,
                       halt_builder: HaltConfBuilder) -> None:

        pri_conf: Dict = self._uiconf['ro']
        pub_conf: Dict = self._uiconf['rw']

        driver = pri_conf.get('driver') or 'libdriver-smartmicro'
        endpoint = pri_conf.get('uri') or 'ummr0a:///dev/ttyS0'
        main_builder.setup_device(driver, endpoint)
        main_builder.setup_scene_event_zone(pri_conf.get('ra-zone'))

        if api_builder is not None:
            args_jnode: Dict = pub_conf['args']

            cam_jnode = args_jnode['camera']
            cam_position_jnode = cam_jnode['position']
            unit_f = cam_position_jnode['forward']
            unit_l = cam_position_jnode['left']
            unit_u = cam_position_jnode['up']
            api_builder.setup_unit_position(unit_f, unit_l, unit_u)

            cam_rotation_jnode = cam_jnode['rotation']
            cam_fl = cam_rotation_jnode['azimuth']
            cam_fu = cam_rotation_jnode['elevation']
            cam_lu = cam_rotation_jnode['roll']
            api_builder.setup_camera_orientation(cam_fl, cam_fu, cam_lu)

            radar_rotation_jnode = args_jnode['radar']['rotation']
            radar_fl = radar_rotation_jnode['azimuth']
            radar_fu = radar_rotation_jnode['elevation']
            radar_lu = radar_rotation_jnode['roll']
            api_builder.setup_radar_orientation(radar_fl, radar_fu, radar_lu)

        halt_lines: List[str] | None = pri_conf.get('haltaction')
        if halt_lines:
            halt_builder.setup_lines(halt_lines)
