from typing import Dict, List

from edge.support import fs


class MainConfBuilder:
    def __init__(self):
        self._jnode = {
            'FileApi': {
                'SceneEvent': list()
            },
            'codec': {
                'Scale': {
                    'MaxWidth': 0,
                    'MaxHeight': 0,
                    'Interpolation': 1
                },
                'Encode': {
                    'UsedThreadMax': 1
                }
            }
        }

    def setup_react(self, spool_dir: str, react_dir: str) -> None:
        self._jnode['FileApi'].update({
            'AddSceneReact': {
                'Dir': fs.join_path(spool_dir, react_dir)
            }
        })

    def setup_event_blobs(self, spool_dir: str, blob_dir: str) -> None:
        self._jnode['FileApi'].update({
            'Store': fs.join_path(spool_dir, blob_dir),
            'FileCountLow': 18,
            'FileCountHigh': 20,
        })

    def setup_event_args(self, spool_dir: str, args_dirs: List[str]) -> None:
        file_api_jnode: Dict = self._jnode['FileApi']
        scene_event_jnode: List[str] = file_api_jnode['SceneEvent']

        for args_dir in args_dirs:
            scene_event_jnode.append(fs.join_path(spool_dir, args_dir))

    def build(self) -> Dict:
        return self._jnode
