from typing import Dict, Final

from edge.webserver.sysconf.dgmod.rg.rg_halt import HaltConfBuilder
from edge.webserver.sysconf.dgmod.rg.rg_main import MainConfBuilder
from edge.webserver.sysconf.dgmod.rg.uiconf import IUiConfProvider


class RtspCameraUiConfProvider(IUiConfProvider):
    def __init__(self, uimod_id: str, uiconf: Dict) -> None:
        self._uimod_id: Final[str] = uimod_id
        self._uiconf: Final[Dict] = uiconf

    def setup_builders(self,
                       main_builder: MainConfBuilder,
                       halt_builder: HaltConfBuilder) -> None:

        pri_conf: Dict = self._uiconf['ro']
        pub_conf: Dict = self._uiconf['rw']

        conf_jnode: Dict = pub_conf['conf']
        video_jnode: Dict = conf_jnode['video']

        # setup enc-source
        flexwatch_conf: Dict | None = pri_conf.get('flexwatch-conf')
        if flexwatch_conf is not None:
            main_builder.setup_enc_source_flexwath(flexwatch_conf)

        else:
            allow_udp = False
            transport = video_jnode.get('transport') or ['RTP/AVP/UDP', 'RTP/AVP/TCP']
            if 'RTP/AVP/UDP' in transport:
                allow_udp = True

            main_builder.setup_enc_source_rtsp(
                conf_jnode['url'], video_jnode['format'], allow_udp)

        # setup dec-source
        dec_source__prefer_gpu: bool = 'hw/auto' == video_jnode.get('decoder', 'hw/auto')
        dec_source__skip_rate: int = video_jnode.get('skip-rate', 1) - 1
        main_builder.setup_dec_source(dec_source__skip_rate, dec_source__prefer_gpu)

        # setup dec-streamer
        main_builder.setup_dec_streamer__bus_blobs_raw('rg{0}-rg{0}-store-raw'.format(self._uimod_id))
        main_builder.setup_dec_streamer__bus_blobs_mono8('rg{0}-rg{0}-store-mono8'.format(self._uimod_id))
        main_builder.setup_dec_streamer__bus_args_grayscale(video_jnode.get('grayscale', False))

        # setup links
        with_blobstore: bool = pri_conf.get('rgbs', False)
        with_mediacodec: bool = pri_conf.get('mcgst-scene', True)
        if with_blobstore:
            main_builder.setup_enc_streamer__bsv_mjpeg(
                blobstore_ident=int(self._uimod_id),
                blobstore_chan='0')

        elif with_mediacodec:
            main_builder.setup_dec_streamer__bus_args([
                f'mg{main_builder.uimod_id}-mcgst{main_builder.uimod_id}-scene'
            ])

        halt_lines = pri_conf.get('haltaction')
        if halt_lines:
            halt_builder.setup_lines(halt_lines)
