from typing import Dict, List

from edge.support import fs


class MainConfBuilder:
    def __init__(self) -> None:
        self._jnode = {
            'services': dict(),
            'scene-event': {
                'max-file-count': 8,
                'endpoints': []
            },
            'Renderer': {
                'profile': {
                    'annotation': {
                        'vehicle': dict()
                    }
                }
            }
        }

    def setup_tracker_react(self, spool_dir: str, react_dir: str) -> None:
        self._jnode.update({
            'Source': {
                'Url': fs.join_path(spool_dir, react_dir)
            }
        })

    def setup_trafficlight_react(self, spool_dir: str, react_dir: str) -> None:
        self._jnode.update({
            'tl-args': [
                fs.join_path(spool_dir, react_dir),
                1
            ]
        })

    def setup_event_blobs(self, spool_dir: str, blobs_dir: str, codec: str) -> None:
        self._jnode.update({
            'image-store': {
                'format': codec,
                'endpoint': fs.join_path(spool_dir, blobs_dir),
                'max-count': 8
            }
        })

    def setup_event_args(self, spool_dir: str, args_dirs: List[str]) -> None:
        jnode: Dict = self._jnode['scene-event']
        endpoints_jnode: List[str] = jnode['endpoints']

        for args_dir in args_dirs:
            endpoints_jnode.append(fs.join_path(spool_dir, args_dir))

    def setup_service_cam(self, intrinsics_path: str, extrinsics_path: str) -> None:
        self._jnode['services'].update({
            'cam': {
                'intrinsics': intrinsics_path,
                'extrinsics': extrinsics_path
            }
        })

    def setup_service_origin(self, conf_path: str) -> None:
        self._jnode['services'].update({
            'origin': conf_path
        })

    def setup_service_datetime(self, conf_path: str) -> None:
        self._jnode['services'].update({
            'datetime': conf_path
        })

    def setup_service_gnss(self, conf_path: str) -> None:
        self._jnode['services'].update({
            'geolocation': conf_path
        })

    def setup_service_stats(self, spool_dir: str, stats_path: str) -> None:
        self._jnode['services'].update({
            'monitoring': {
                'path': fs.join_path(spool_dir, stats_path),
                'tout': 30
            }
        })

    def setup_image_footer(self, image_w: int, footer: Dict) -> None:
        self._jnode['Renderer']['profile'].update({
            'width': image_w,
            'footer': footer
        })

    def setup_image_anno_lr(self, anno_conf: Dict) -> None:
        """
        :param anno_conf:
            http-streaming/0.json#/rw/live/renderer/vehicle
            rtsp-streaming/0.json#/rw/live/renderer/vehicle
        """
        self._jnode['Renderer']['profile']['annotation']['vehicle'].update({
            'lr': {
                'plate': anno_conf.get('video-plate', 'never'),
                'dir': anno_conf.get('video-dir', 'never'),
                'lane': anno_conf.get('video-lane', 'never'),
                'trace': anno_conf.get('video-plate-trace', 'never'),
                'center': anno_conf.get('video-plate-center', 'never')
            }
        })

    def setup_image_anno_ra(self, anno_conf: Dict) -> None:
        """
        :param anno_conf:
            http-streaming/0.json#/rw/live/renderer/vehicle
            rtsp-streaming/0.json#/rw/live/renderer/vehicle
        """
        self._jnode['Renderer']['profile']['annotation']['vehicle'].update({
            'ra': {
                'target': anno_conf.get('radar-target', 'never'),
                'vehicle': anno_conf.get('radar-vehicle', 'never'),
                'speed': anno_conf.get('radar-speed', 'never'),
                'len': anno_conf.get('radar-length', 'never'),
                'dir': anno_conf.get('radar-dir', 'never')
            }
        })

    def setup_spotspeed(self, speed_limit: int, assert_speed: int) -> None:
        self._jnode['Renderer'].update({
            'YellowSpeedThreshold': speed_limit,
            'RedSpeedThreshold': assert_speed
        })

    def setup_roadway(self, conf: Dict) -> None:
        """
        :param conf:
            /etc/edge/setupstore/admin/actions/pipeline/0/lane-enforcement/0.json#/rw/args
        """
        self._jnode['road-mask'] = {
            'lanes': conf['lanes'],
            'scene-wh': conf['scene-wh']
        }

    def build(self) -> Dict:
        return self._jnode
