from typing import Dict

from edge.support import fs


# noinspection PyClassHasNoInit
class CtrlUiconf:
    _CONF_PATH = '/etc/edge/setupstore/admin/actions/controller.json'

    @classmethod
    def get_systest(cls) -> bool:
        if fs.path_exists(cls._CONF_PATH):
            uiconf = fs.load_json3(cls._CONF_PATH)
            return uiconf['ro'].get('systest', False)

        return False

    @classmethod
    def set_systest(cls, systest: bool) -> None:
        if fs.path_exists(cls._CONF_PATH):
            uiconf = fs.load_json3(cls._CONF_PATH)
            uiconf['ro']['systest'] = systest
            fs.write_json_file(uiconf, cls._CONF_PATH, pretty=True)
        else:
            uiconf = {
                'ro': {
                    'systest': systest
                }
            }

            fs.write_json_file(uiconf, cls._CONF_PATH, pretty=True)

    @classmethod
    def get_contrast_vehicle(cls) -> Dict:
        if fs.path_exists(cls._CONF_PATH):
            uiconf = fs.load_json3(cls._CONF_PATH)
            return uiconf['ro'].get('contrast', {}).get('vehicle', {})

        return dict()

    @classmethod
    def get_contrast_scene(cls) -> Dict:
        if fs.path_exists(cls._CONF_PATH):
            uiconf = fs.load_json3(cls._CONF_PATH)
            return uiconf['ro'].get('contrast', {}).get('scene', {})

        return dict()

    @classmethod
    def get_bylaws(cls) -> str:
        if fs.path_exists(cls._CONF_PATH):
            uiconf = fs.load_json3(cls._CONF_PATH)
            return uiconf['ro'].get('bylaws', 'default')

        return 'default'

    @classmethod
    def set_bylaws(cls, bylaws: str) -> None:
        if fs.path_exists(cls._CONF_PATH):
            uiconf = fs.load_json3(cls._CONF_PATH)
            uiconf['ro']['bylaws'] = bylaws
            fs.write_json_file(uiconf, cls._CONF_PATH, pretty=True)
        else:
            uiconf = {
                'ro': {
                    'bylaws': bylaws
                }
            }

            fs.write_json_file(uiconf, cls._CONF_PATH, pretty=True)
