# coding=utf-8

__all__ = [
    'IitCaUiconf',
    'IitCaEndUserUiconf'
]

from typing import Dict, Optional, Self

from edge.webserver.sysconf import ph
from edge.webserver.sysconf.uiconf.uiconf2 import UiConf2


class IitCaUiconf(UiConf2):
    def __init__(self, conf: UiConf2):
        super().__init__(conf.raw)

    @classmethod
    def create(cls, factory_conf: Optional[Dict] = None) -> Self:
        service_uiref = factory_conf['ref']
        iit_ca_unit_name = 'ІІТ ЦСК'

        conf_jnode = {
            'pri': {
                'schema-path': '/opt/edge/share/setupstore/ui/iit-ca-schema.json',
                'lock-path': f'/var/run/edge/{service_uiref}',
                'save-loaded-certs': True
            },
            'pub': {
                'type': ph.UI_TYPE_IIT_CA,
                'conf': {
                    'name': iit_ca_unit_name,
                    'proxy': {
                        'enable': True,
                        'host': '10.100.27.101',
                        'port': 3128,
                        'auth-type': 'anonymous',
                        'auth-user': '',
                        'auth-pass': ''
                    },
                    'cmp': {
                        'host': 'ca.mvs.gov.ua',
                        'port': 80
                    },
                    'tsp': {
                        'host': 'ca.mvs.gov.ua',
                        'port': 80
                    },
                    'crl': {
                        'enable': False,
                        'full-and-delta-crls': False,
                        'only-own-crls': False
                    },
                    'ocsp': {
                        'enable': False,
                        'host': '',
                        'port': 80
                    }
                }
            }
        }

        return cls(UiConf2(conf_jnode))

    @property
    def proxy_pass(self) -> Optional[str]:
        return self.pub_conf['proxy'].get('auth-pass')

    @property
    def save_load_certs(self) -> bool:
        return self.pri.get('save-loaded-certs', True)


class IitCaEndUserUiconf(UiConf2):
    def __init__(self, conf: UiConf2):
        super().__init__(conf.raw)

    @classmethod
    def create(cls, factory_jnode: Optional[Dict] = None) -> Self:
        name = factory_jnode.get('name', 'ІІТ Користувач ЦСК')
        factory_conf = factory_jnode['conf']
        media_conf = factory_conf['media']
        model = media_conf['model']
        media_serial = media_conf['serial']
        media_pass = media_conf['password']
        iit_ca_uimod_ref = factory_conf['parent-ref']
        zip_file_name = factory_conf.get('zip-name', '')
        sign_active = factory_conf.get('active', False)

        conf_jnode = {
            'pri': {
                'model': model,
                'serial': media_serial,
                'password': media_pass,
                'lock-path': f'/var/run/edge/{iit_ca_uimod_ref}',
                'csr-name': zip_file_name
            },
            'pub': {
                'type': ph.UI_TYPE_IIT_CA_END_USER,
                'conf': {
                    'name': name,
                    'enabled': sign_active
                }
            }
        }

        return cls(UiConf2(conf_jnode))

    @property
    def model(self) -> str:
        return self.pri['model']

    @property
    def serial(self) -> str:
        return self.pri['serial']

    @property
    def password(self) -> str:
        return self.pri['password']

    @property
    def csr_name(self) -> str:
        return self.pri.get('csr-name', '')

    @property
    def crt_stats(self) -> Optional[Dict]:
        return self.pri.get('crt')

    def set_crt_stats(self, stats: Dict) -> None:
        self.pri['crt'] = stats
