from typing import Dict, List, Optional

from edge.support import fs


# noinspection PyClassHasNoInit
class PipelineUiconf:
    _DEFCONF_PATH = '/opt/edge/share/setupstore/ui/pipeline-jnode.json'
    CONF_PATH = '/etc/edge/setupstore/admin/actions/pipeline.json'

    PTYPE_TERM_RTSP = '1'   # rtsp-videosource; radar absent
    PTYPE_TERM_RADAR = '2'  # gige-or-usb-videosource; radar present
    PTYPE_TERM_MV = '3'     # gige-or-usb-videosource; radar absent

    @classmethod
    def load_conf(cls, term_pid: Optional[str] = None) -> List[Dict]:
        fs.create_jnode_from_default(cls._DEFCONF_PATH, cls.CONF_PATH)
        terms_info = fs.load_json3(cls.CONF_PATH, default=None)
        if term_pid:
            terms_info = [x for x in terms_info if x['id'] == term_pid]

        return terms_info

    @classmethod
    def save_conf(cls, terms_info: List[Dict]) -> None:
        fs.write_json_file(terms_info, cls.CONF_PATH, pretty=True)
