import os
import subprocess
import ujson

from typing import Dict

from edge import eventlog
from edge.support import net

UI_NETWORK_JNODE_FILEPATH = '/etc/edge/setupstore/admin/actions/network.json'
MANAGEMENT_MAC_FILEPATH = '/etc/network/management-mac'


# noinspection PyClassHasNoInit
class RtspStreamingUiconf:
    def __init__(self, ui_rw_jnode: Dict):
        self._ui_rw_jnode = ui_rw_jnode

    @property
    def rtsp_url(self) -> str:
        try:
            with open(UI_NETWORK_JNODE_FILEPATH) as json_file:
                ui_network_jnode = ujson.load(json_file)
            mac_address = ui_network_jnode['ro']['mac']

        except IOError as x:
            if x.errno == 2:
                if os.path.exists(MANAGEMENT_MAC_FILEPATH):
                    with open(MANAGEMENT_MAC_FILEPATH, 'r') as mgmt_mac_file:
                        mac_address = mgmt_mac_file.readlines()[0].strip()
                else:
                    raise

            else:
                eventlog.log_d('fail: read network settings')
                raise IOError

        ifname = net.get_ifname(mac_address)  # TODO: check if ifname is not None
        try:
            shell_params = f"ip addr show {ifname} | grep -v '127.0.0.1' | grep -w inet | awk '{{print $2}}'"
            shell_result = subprocess.run(shell_params, capture_output=True, text=True, encoding='ascii', shell=True)
            ip_addr = shell_result.stdout.split('/')[0]
        except subprocess.CalledProcessError as x:
            eventlog.log_e('fail: <{}> (returncode:{}, stdout:<{}>, stderr:<{}>)'.format(
                x.cmd, x.returncode, x.stdout, x.stderr))
            raise RuntimeError

        tcp_port = str(self._ui_rw_jnode['streaming']['tcp-port'])
        http_format = self._ui_rw_jnode['live']['videostream']['format']
        if http_format == 'mjpeg':
            rtsp_format = 'mjpeg'
        elif http_format == 'h264-cqp' or 'h264-cbr':
            rtsp_format = 'h264'
        else:
            eventlog.log_d(f'fail: unsupported (format:{http_format})')
            raise ValueError

        rtsp_url = f'rtsp://{ip_addr}:{tcp_port}/live-{rtsp_format}'

        return rtsp_url
