from typing import Dict, Optional

from edge.support import fs
from edge.types.exceptions import InternalError


class UiConf2:
    def __init__(self, jnode: Dict):
        self._raw = jnode

    @property
    def raw(self) -> Dict:
        return self._raw

    @property
    def pri(self) -> Dict:
        return self.raw['pri']

    @property
    def pub(self) -> Dict:
        return self.raw['pub']

    @property
    def pub_conf(self) -> Dict:
        return self.pub['conf']

    @property
    def pub_name(self) -> str:
        return self.pub_conf['name']

    def set_pub(self, jnode: Dict) -> None:
        self.raw['pub'] = jnode

    def set_pri(self, jnode: Dict) -> None:
        self.raw['pri'] = jnode

    def set_ver(self, version: str) -> None:
        self.pri['ver'] = version

    @property
    def ver(self) -> str:
        ver = self.pri.get('ver')
        if ver is None:
            raise InternalError

        return ver

    def set_pub_conf(self, jnode: Dict) -> bool:
        changed = self.pub['conf'] != jnode
        self.pub['conf'] = jnode
        return changed

    def set_pub_name(self, name: str) -> bool:
        changed = self.pub_conf['name'] != name
        self.pub_conf['name'] = name
        return changed

    @property
    def lock_path(self) -> str:
        return self.pri['lock-path']

    @property
    def _schema_path(self) -> Optional[str]:
        return self.pri.get('schema-path')

    @property
    def terminal_ref(self) -> Optional[str]:
        return self.pri.get('terminal-ref')

    def validate(self) -> None:
        """
        :raises ParamsError
        """
        schema_path = self._schema_path
        if schema_path:
            fs.validate_json(self.pub, schema_path)
