from typing import Dict, Self

from edge.support import fs


class EsUimod:
    _SCHEMA_PATH = '/opt/edge/share/setupstore/ui/eventstore-ch-schema.json'
    _DEFCONF_PATH = '/opt/edge/share/setupstore/ui/eventstore-ch-jnode.json'
    _CONF_DIR = '/etc/edge/setupstore/admin/actions/eventstore'

    def __init__(self, uimod_ref: str):
        self._ref = uimod_ref
        self._ch_id = self._ref.split('.')[-1]

    @classmethod
    def create(cls, uimod_ref: str) -> Self:
        return cls(uimod_ref)

    @property
    def schema_path(self) -> str:
        return self._SCHEMA_PATH

    @property
    def _conf_path(self) -> str:
        return fs.join_path(self._CONF_DIR, self._ch_id, '0.json')

    def load_conf(self, **kwargs) -> Dict:
        """
        :raise FsError:
        """
        return fs.load_json3(self._conf_path, **kwargs)

    def save_conf(self, conf_jnode: Dict) -> None:
        fs.write_json_file(conf_jnode, self._conf_path, pretty=True)

    def install(self, name: str, bschan_64k: str) -> Dict:
        fs.create_jnode_from_default(self._DEFCONF_PATH, self._conf_path)
        uiconf = self.load_conf()

        uiconf_pub: Dict = uiconf['rw']
        uiconf_pub['args']['name'] = name

        uiconf_pri: Dict = uiconf['ro']
        uiconf_pri['channel'] = bschan_64k

        self.save_conf(uiconf)
        return uiconf

    def uninstall(self) -> None:
        uiconf_dir = fs.join_path(self._CONF_DIR, self._ch_id)
        fs.silent_rm_r(uiconf_dir)
