__all__ = [
    'DB_SECTION',
    'DB_KEY_TESTPACK',
    'mk_test_db_key',
    'mk_test_db_glob',
    'mk_ref',
    'Services',
    'mk_vs_bbox4',
    'get_rcs',
    'SIMULATOR_TYPE_EKTSDG',
    'SIMULATOR_TYPE_IS24D',
    'SIMULATOR_TYPE_SAPSAN3_1',
    'SIMULATOR_TYPE_SAPSAN3_2'
]

from typing import List, Optional, Tuple

from edge import eventlog
from edge.services import db_value
from edge.services.types.tz import ITzService
from edge.webserver.sysconf import ph
from edge.webserver.sysconf.api_terminal import TerminalApi
from edge.webserver.systest.exception import TestError


DB_SECTION = '/sys/test'
DB_KEY_TESTPACK = '/'


def mk_vs_bbox4(ui_pid: str, rn_bbox4: Optional[List[int]]) -> Optional[List[int]]:
    if rn_bbox4 is None:
        return None

    from edge.webserver.sysconf.dgstats.renderer import RnDgStats

    vs_wh = ph.CamDgStats.create(ui_pid).get_stream_wh()
    rn_wh_dh = RnDgStats.create(ui_pid).get_stream_wh_dh()

    if None in [vs_wh, rn_wh_dh]:
        raise Exception

    vs_w, vs_h = vs_wh
    rn_w, rn_h, rn_dh = rn_wh_dh

    scale_factor = vs_w / float(rn_w)
    vs_bbox4 = [int(x * scale_factor) for x in rn_bbox4]
    return vs_bbox4


def mk_test_db_key(session_ref: str, test_ref: Tuple[str, str]) -> str:
    return f'/{session_ref}/{test_ref[0]}/{test_ref[1]}'


def mk_test_db_glob(session_ref: str) -> str:
    return f'\/{session_ref}\/.*'


def mk_ref(known_refs: List[str] = ()) -> str:
    # TODO: delete this
    import uuid
    return str(uuid.uuid4())
    #

    # refs = set(known_refs)
    # ref = len(refs)
    # while str(ref) in refs:
    #     ref += 1
    #
    # return str(ref)


class Services:
    def __init__(self, tz_service: ITzService, db_service: db_value.DbValueApi):
        self.tz = tz_service
        self.db = db_service


SIMULATOR_TYPE_EKTSDG = 'EKTSDG'
SIMULATOR_TYPE_IS24D = 'IS-24D'
SIMULATOR_TYPE_SAPSAN3_1 = 'SAPSAN3-1'
SIMULATOR_TYPE_SAPSAN3_2 = 'SAPSAN3-2'


def get_rcs(ui_pid: str, simulator_type: str = SIMULATOR_TYPE_EKTSDG, systest_active: bool = False) -> Optional[int]:
    def_key = 'def'
    simulator_to_def_rcs = {
        '29': {
            def_key: -3155,
            SIMULATOR_TYPE_EKTSDG: -3155,
            SIMULATOR_TYPE_IS24D: None,
            SIMULATOR_TYPE_SAPSAN3_1: -6000,
            SIMULATOR_TYPE_SAPSAN3_2: -3155
        },
        '30': {
            def_key: -3155,
            SIMULATOR_TYPE_EKTSDG: -3155,
            SIMULATOR_TYPE_IS24D: -3155,
            SIMULATOR_TYPE_SAPSAN3_1: None,
            SIMULATOR_TYPE_SAPSAN3_2: -3155
        },
        '39': {
            def_key: -1301,
            SIMULATOR_TYPE_EKTSDG: -1301,
            SIMULATOR_TYPE_IS24D: None,
            SIMULATOR_TYPE_SAPSAN3_1: None,
            SIMULATOR_TYPE_SAPSAN3_2: None
        },
        'r3d': {
            def_key: 14,
            SIMULATOR_TYPE_EKTSDG: 14,
            SIMULATOR_TYPE_IS24D: 14,
            SIMULATOR_TYPE_SAPSAN3_1: 14,
            SIMULATOR_TYPE_SAPSAN3_2: 14
        }
    }

    ra_type = TerminalApi.get_ra_type(ui_pid)
    if ra_type is None:
        eventlog.log_e('fail: ra-type is mising')
        raise TestError

    index = simulator_type if systest_active else def_key

    ra_conf = simulator_to_def_rcs.get(ra_type, {})
    rcs = ra_conf.get(index) or ra_conf[def_key]

    if rcs is None:
        eventlog.log_e('fail: get rcs')
        raise TestError

    return rcs
