from typing import Dict, Optional, Self

from edge import eventlog
from edge.types.exceptions import JnodeError

from eventstore.services.types import ApiArgs, StoreArgs, ReactArgs, EventArgs, StatsArgs


class KernelArgs:
    def __init__(self, api: ApiArgs, react: ReactArgs, store: StoreArgs, event: Optional[EventArgs], stats: StatsArgs):
        self.api = api
        self.react = react
        self.store = store
        self.event = event
        self.stats = stats

    @classmethod
    def unpack(cls, jnode: Dict) -> Self:
        """
        :param jnode:
            main.conf#/

        :raise JnodeError:
        """
        store = StoreArgs.unpack(jnode)

        # unpack #/api
        endpoint = jnode.get('api')
        api = ApiArgs.unpack(endpoint)

        # unpack #/fact-react
        react_jnode = jnode.get('fact-react', [])
        if type(react_jnode) != list:
            eventlog.log_e_param('#/fact-react')
            raise JnodeError

        react = ReactArgs.unpack(react_jnode)

        # unpack #/fact-event
        event_jnode = None if store.systest else jnode.get('fact-event')
        event = EventArgs.unpack(event_jnode) if event_jnode else None

        # unpack #/monitoring
        stats_jnode = jnode.get('monitoring')
        if not stats_jnode:
            eventlog.log_e_param('#/monitoring')
            raise JnodeError

        stats = StatsArgs.unpack(stats_jnode)
        return cls(api, react, store, event, stats)
