from PIL import Image
from typing import List, Self

from edge.types.image import ImRgba
from edge.support.queue import JsonFileQueue, ImageFileQueue

from renderer._args import EventArgs, ImStoreArgs


class SceneEventConsumer:
    _STATUS_SUCCESS = 1
    _STATUS_FAILURE = 2

    def __init__(self, twisted_reactor, codec: str,
                 json_file_queue_list: List[JsonFileQueue],
                 image_file_queue: ImageFileQueue):

        self._twisted_reactor = twisted_reactor
        self._target_codec = codec
        self._json_file_queue_list = json_file_queue_list
        self._image_file_queue = image_file_queue

    def consume_scene(self, scene_ts: int, blob: ImRgba) -> None:
        w, h = blob.wh
        with Image.frombuffer(
                'RGBA', (w, h + blob.dh), blob.buffer,
                'raw', 'RGBA', blob.stride, 1) as pil_img:

            image_ref = self._image_file_queue.add_image(scene_ts, pil_img)

            event_args_jnode = {
                'Ts': image_ref.ts,
                'Status': SceneEventConsumer._STATUS_SUCCESS,  # todo
                'SizeWh': pil_img.size,
                'Url': image_ref.url,
                'Codec': image_ref.codec
            }

            list(map(lambda q: q.add_jnode(event_args_jnode), self._json_file_queue_list))

    @classmethod
    def create(cls, twisted_reactor, scene_args: EventArgs, store_args: ImStoreArgs) -> Self:
        json_file_queue_list = [JsonFileQueue(queue_args) for queue_args in scene_args.json_queue_args_list]

        image_file_queue = ImageFileQueue(
            twisted_reactor, store_args.codec,
            store_args.image_queue_args.endpoint,
            store_args.image_queue_args.min_file_count,
            store_args.image_queue_args.max_file_count,
            scene_args.log_level)

        return cls(twisted_reactor, store_args.codec, json_file_queue_list, image_file_queue)
