import time
import os

from edge.support import fs
from edge import eventlog
from renderer._args import StatsServiceArgs


class WhCounters:
    def __init__(self):
        self.w = 0
        self.h = 0
        self.dh = 0

    def im_size(self, w: int, h: int, dh: int) -> None:
        self.w = w
        self.h = h
        self.dh = dh


class IStatService:
    @property
    def wh(self) -> WhCounters:
        raise NotImplementedError


class _StubStatService(IStatService):
    def __init__(self):
        IStatService.__init__(self)

        self._wh = WhCounters()

    @property
    def wh(self) -> WhCounters:
        return self._wh


class _StatService(IStatService):
    def __init__(self, twisted_reactor, args: StatsServiceArgs):
        """
        :param twisted_reactor:
            twisted.internet.interfaces.IReactorTime
        :param args:
        """
        IStatService.__init__(self)

        self._wh = WhCounters()

        self._stat_abspath = args.stat_filepath
        self._stat_tmp_abspath = args.stat_filepath + '.tmp'
        self._stat_tout_sec = args.stat_tout_sec

        if args.stat_tout_sec > 0:
            twisted_reactor.callLater(args.stat_tout_sec, self._log_counters, twisted_reactor)

    @property
    def wh(self) -> WhCounters:
        return self._wh

    def _log_counters(self, twisted_reactor) -> None:
        self._report_counters()
        twisted_reactor.callLater(self._stat_tout_sec, self._log_counters, twisted_reactor)

    def _report_counters(self) -> None:
        sys_utc_time = time.time()

        stat = {
            "stat": {
                "mtime": [time.strftime('%d.%m.%Y %H:%M:%S', time.gmtime(sys_utc_time)), int(round(sys_utc_time))],
                "tout-sec": self._stat_tout_sec
            },
            "stream": {
                "wh": [self.wh.w, self.wh.h + self.wh.dh],
                "dh": self.wh.dh
            }
        }

        try:
            fs.write_json_file(stat, self._stat_tmp_abspath, True)
            os.rename(self._stat_tmp_abspath, self._stat_abspath)
        except IOError as x:
            eventlog.log_w(f'fail: write <{self._stat_abspath}> ({x})')


def init_service(twisted_reactor, args: StatsServiceArgs) -> IStatService:
    """
    :param twisted_reactor:
        twisted.internet.interfaces.IReactorTime
    :param args:
    """
    if args.stat_tout_sec:
        stat_service = _StatService(twisted_reactor, args)
    else:
        stat_service = _StubStatService

    return stat_service


# noinspection PyUnusedLocal
def fini_service(service: IStatService):
    pass
