from typing import Dict

from edge import eventlog
from edge.types.exceptions import JnodeError
from edge.support.mask import MaskConf, decode__mask_region


def decode__mask_conf(zone_jnode: Dict) -> MaskConf:
    """
    :raise JnodeError:
    """
    area_jnode = zone_jnode.get('area')
    if type(area_jnode) != dict:
        eventlog.log_e_param('#/area')
        raise JnodeError

    # deserialize #/area/mask
    mask = decode__mask_region(area_jnode.get('mask'))

    # deserialize #/area/scene-wh
    scene_wh_jnode = area_jnode.get('scene-wh')
    if not type(scene_wh_jnode) in [list, tuple] or len(scene_wh_jnode) != 2:
        eventlog.log_e_param('#/area/scene-wh')
        raise JnodeError

    w, h = scene_wh_jnode
    if not (int == type(w) and 640 <= w <= 65535):
        eventlog.log_e_param('#area/scene-wh/0')
        raise JnodeError

    if not (int == type(h) and 480 <= h <= 65535):
        eventlog.log_e_param('#area/scene-wh/1')
        raise JnodeError

    return MaskConf(mask, (w, h))
