from typing import Final, Optional, Self

from edge.services.types.cam import ICamService
from edge.services.types.about import IAboutService
from edge.services.types.tz import ITzService
from edge.support.service import ServicePack

from speedcontrol.services.types.clock import IClockService
from speedcontrol.services.types.plateinfo import IPlateInfoService
from speedcontrol.services.types.raas import IRaasService
from speedcontrol.services.types.trainer import ITrainerService
from edge.services.types.vs import IVsService


class DetectorDeps:
    def __init__(
            self,
            tz_service: ITzService,
            vs_service: IVsService,
            raas_service: IRaasService,
            trainer_service: ITrainerService,
            about_service: IAboutService,
            plate_info_service: IPlateInfoService,
            cam_service: Optional[ICamService],
            clock_service: IClockService):

        self.tz: Final[ITzService] = tz_service
        self.vs: Final[IVsService] = vs_service
        self.raas: Final[IRaasService] = raas_service
        self.trainer: Final[ITrainerService] = trainer_service
        self.about: Final[IAboutService] = about_service
        self.plate_info: Final[IPlateInfoService] = plate_info_service
        self.cam: Final[ICamService | None] = cam_service
        self.clock: Final[IClockService] = clock_service

    @classmethod
    def create(cls, services: ServicePack) -> Self:
        return cls(
            services.get(ITzService),
            services.get(IVsService),
            services.get(IRaasService),
            services.get(ITrainerService),
            services.get(IAboutService),
            services.get(IPlateInfoService),
            services.try_get(ICamService),
            services.get(IClockService))
