from typing import Dict

from edge import eventlog
from edge.image.jnode.types import decode__im_builder_conf
from edge.types.identity import Identity
from edge.types.exceptions import JnodeError

from speedcontrol.detectors.base.types.conf import DetectorConf


# noinspection PyUnusedLocal
def decode__detector_conf(identity: Identity, jnode: Dict, def_footer_mapping: Dict) -> DetectorConf:
    """
    :param identity:
    :param jnode:
        /etc/edge/speedcontrol.x/transit.json#/
    :param def_footer_mapping:

    :raise JnodeError:
    """
    # deserialize #/images
    images_jnode = jnode.get('images')
    if type(images_jnode) != dict:
        eventlog.log_e('fail: #/images')
        raise JnodeError

    # deserialize #/images/best
    best_waypoint_jnode = images_jnode.get('best')
    if type(best_waypoint_jnode) != dict:
        eventlog.log_e_param('#/images/best')
        raise JnodeError

    im_engine = decode__im_builder_conf(best_waypoint_jnode)

    # deserialize #/systest
    systest = jnode.get('systest', False)

    # deserialize #/enable-overview
    enable_overview = jnode.get('enable-overview', False)

    return DetectorConf(def_footer_mapping, im_engine, systest, enable_overview)
