from typing import List, Optional, Dict, Union

from edge.image.types.types import EventMeasImBuilderConf
from edge.types.identity import Identity
from edge.support.mask import MaskConf

from speedcontrol.detectors.types.conf import VideoConf
from edge.support.schedule import ScheduleConf
from speedcontrol.events.ticket import TicketInfo


class RuleLimits:
    def __init__(self, vehicle_types: List[str]):
        self.vehicle_types = vehicle_types


class RuleConf:
    def __init__(self, name: str, enabled: bool, schedule: Optional[ScheduleConf], ticket: TicketInfo,
                 limits: RuleLimits, footer_defvars: Dict):
        self.name = name
        self.enabled = enabled
        self.schedule = schedule
        self.ticket = ticket
        self.limits = limits
        self.footer_defvars = footer_defvars


class DetectorConf:
    def __init__(self,
                 identity: Identity,
                 name: str,
                 store: str,
                 rules: List[RuleConf],
                 scene: MaskConf,
                 im_engine: EventMeasImBuilderConf,
                 video_args: Optional[VideoConf],
                 min_ts_dist_usec: Union[float, int] = 1e6):

        self.identity = identity
        self.name = name
        self.store = store
        self.rules = rules
        self.scene = scene
        self.im_engine = im_engine
        self.video = video_args
        self.min_ts_dist_usec = min_ts_dist_usec
