from typing import Dict

from edge import eventlog
from edge.image.jnode.types import decode__im_builder_conf
from edge.types.exceptions import JnodeError

from speedcontrol.detectors.buslane.types.conf import DetectorConf
from speedcontrol.detectors.buslane.types.conf import RuleLimits, RuleConf
from speedcontrol.detectors.types.jnode.conf import decode__base_rule_conf, decode__video_conf
from speedcontrol.conf.jnode.mask import decode__mask_conf
from edge.types.identity import Identity


def decode__rule_limits(jnode: Dict) -> RuleLimits:
    """
    :raise JsonError:
    """
    # deserialize #/classes
    types_jnode = jnode.get('classes', [])

    if type(types_jnode) != list:
        eventlog.log_e_param('#/classes')
        raise JnodeError

    elif len(types_jnode) == 0:
        eventlog.log_e_param('#/classes')
        raise JnodeError

    for v_type in types_jnode:
        if not type(v_type) in [str]:
            eventlog.log_e_param('#/classes/%s' % v_type)
            raise JnodeError

    return RuleLimits(types_jnode)


def decode__rule_conf(rule_jnode: Dict, footer_defvars: Dict) -> RuleConf:
    """
    :raise JsonError:
    """
    if type(rule_jnode) != dict:
        eventlog.log_e_param('#/')
        raise JnodeError

    # deserialize #/name
    # deserialize #/enabled
    # deserialize #/schedule
    # deserialize #/ticket
    rule = decode__base_rule_conf(rule_jnode)

    # deserialize #/limits
    limits_jnode = rule_jnode.get('limits')
    if type(rule_jnode) != dict:
        eventlog.log_e_param('#/limits')
        raise JnodeError

    limits = decode__rule_limits(limits_jnode)
    return RuleConf(rule.name, rule.enabled, rule.schedule, rule.ticket, limits, footer_defvars)


def decode__detector_conf(identity: Identity, jnode: Dict, footer_defvars: Dict) -> DetectorConf:
    """
    :raise JsonError:
    """
    # deserialize #/name
    name = jnode.get('name')
    if not type(name) in [str] or len(name) > 32:
        eventlog.log_e_param('#/name')
        raise JnodeError

    # deserialize #/store
    store = jnode.get('store')
    if not (type(store) in [str] and 1 <= len(store) <= 2):
        eventlog.log_e_param('#/store')
        raise JnodeError

    # deserialize #/rules
    rules_jnode = jnode.get('rules')
    if type(rules_jnode) != list:
        eventlog.log_e_param('#/rules')
        raise JnodeError

    # deserialize #/scene
    scene_jnode = jnode.get('scene')
    if type(scene_jnode) != dict:
        eventlog.log_e_param('#/scene')
        raise JnodeError
    scene = decode__mask_conf(scene_jnode)

    rules = []
    for rule_jnode in rules_jnode:
        rule = decode__rule_conf(rule_jnode, footer_defvars)
        if rule.enabled:
            rules.append(rule)

    # deserialize #/images
    images_jnode = jnode.get('images')
    if type(images_jnode) != dict:
        eventlog.log_e_param('#/images')
        raise JnodeError

    im_engine = decode__im_builder_conf(images_jnode['violation'])

    # deserialize #/videos/measuments
    video_jnode = jnode.get('videos', {}).get('violation')
    if video_jnode is not None:
        if video_jnode.get('enabled', False):
            video_args = decode__video_conf(video_jnode)
        else:
            video_args = None
    else:
        video_args = None

    min_ts_dist_usec = jnode.get('min-image-dist', 1e3) * 1e3

    return DetectorConf(identity, name, store, rules, scene, im_engine,
                        video_args, min_ts_dist_usec)
