from typing import Dict, List

from edge import eventlog
from edge.image.jnode.types import decode__im_builder_conf
from edge.types.exceptions import JnodeError

from speedcontrol.detectors.seatbelt.types.conf import DetectorConf, SeatConf
from edge.types.identity import Identity
from speedcontrol.events.jnode.ticket import decode__ticket


def decode__seat_conf(jnode: Dict) -> SeatConf:
    min_weight = jnode.get('min-weight', 0)
    enabled = jnode['enabled']
    privacy_mask = jnode['privacy-mask']
    return SeatConf(enabled, privacy_mask == 'violation-absent', min_weight)


def decode__detector_conf(identity: Identity, jnode: Dict, def_footer_mapping: Dict) -> DetectorConf:
    """
    :raise JsonError:
    """
    # deserialize #/name
    name = jnode.get('name')
    if not type(name) in [str] or len(name) > 32:
        eventlog.log_e_param('#/name')
        raise JnodeError

    # deserialize #/store
    store = jnode.get('store')
    if not (type(store) in [str] and 1 <= len(store) <= 2):
        eventlog.log_e_param('#/store')
        raise JnodeError

    # deserialize #/seats
    seats_jnode = jnode.get('seats')
    if type(seats_jnode) != list:
        eventlog.log_e_param('#/seats')
        raise JnodeError

    seat_confs: List[SeatConf] = list()

    for seat_jnode in seats_jnode:
        seat_confs.append(decode__seat_conf(seat_jnode))

    # deserialize #/images
    images_jnode = jnode.get('images')
    if type(images_jnode) != dict:
        eventlog.log_e_param('#/images')
        raise JnodeError

    im_engine = decode__im_builder_conf(images_jnode['violation'])
    ticket = decode__ticket(jnode.get('ticket', dict()))

    return DetectorConf(identity, name, store, seat_confs, ticket, im_engine, def_footer_mapping)
