from typing import Optional, Self

from edge.services.types.cam import ICamService
from edge.services.types.about import IAboutService
from edge.services.types.tz import ITzService
from edge.support.service import ServicePack
from speedcontrol.services.types.clock import IClockService
from edge.services.types.vs import IVsService


class RuleDeps:
    def __init__(self,
                 tz_service: ITzService,
                 vs_service: IVsService,
                 about_service: IAboutService,
                 clock_service: IClockService,
                 cam_service: Optional[ICamService]):

        self.tz = tz_service
        self.vs = vs_service
        self.about = about_service
        self.clock = clock_service
        self.cam_service = cam_service

    @classmethod
    def create(cls, services: ServicePack) -> Self:
        return cls(
            services.get(ITzService),
            services.get(IVsService),
            services.get(IAboutService),
            services.get(IClockService),
            services.try_get(ICamService))


class DetectorDeps(RuleDeps):
    pass
