from typing import Dict, Final, List, Optional, Set

from edge.image.types.types import EventMeasImBuilderConf
from edge.support.schedule import ScheduleConf
from edge.types.identity import Identity

from speedcontrol.detectors.types.conf import VideoConf
from speedcontrol.events.ticket import TicketInfo


class RuleLimits:
    def __init__(self,
                 lanes: Set[str],
                 speed_limit: int,
                 max_overspeed: int,
                 vehicle_types: Set[str]):
        self.lanes: Final[Set[str]] = lanes
        self.speed_limit: Final[int] = speed_limit
        self.max_overspeed: Final[int] = max_overspeed
        self.vehicle_types: Final[Set[str]] = vehicle_types


class RuleConf:
    def __init__(self,
                 identity: Identity,
                 detector_name: str,
                 rule_name: str,
                 enabled: bool,
                 schedule: Optional[ScheduleConf],
                 ticket: TicketInfo,
                 limits: RuleLimits,
                 footer_defvars: Dict):

        self.identity = identity
        self.detector_name = detector_name
        self.rule_name = rule_name
        self.enabled = enabled
        self.schedule = schedule
        self.ticket = ticket
        self.limits = limits
        self.footer_defvars = footer_defvars


class DetectorConf:
    def __init__(self,
                 identity: Identity,
                 name: str,
                 store: str,
                 rules: List[RuleConf],
                 im_engine: EventMeasImBuilderConf,
                 video_args: Optional[VideoConf]):
        self.identity = identity
        self.name = name
        self.store = store
        self.rules = rules
        self.im_engine = im_engine
        self.video = video_args
