from typing import Dict, List, Optional, Set, Final

from edge.image.types.types import EventMeasImBuilderConf

from speedcontrol.detectors.types.conf import VideoConf
from edge.support.schedule import ScheduleConf
from speedcontrol.events.ticket import TicketInfo
from edge.types.identity import Identity


class LimitsConf:
    def __init__(self,
                 lanes: Set[str],
                 speed_limit: int,
                 over_speed: int,
                 vehicle_types: List[str]):

        self.lanes: Final[Set[str]] = lanes
        self.speed_limit = speed_limit
        self.over_speed = over_speed
        self.vehicle_types = vehicle_types


class RuleConf:
    def __init__(self,
                 name: str,
                 enabled: bool,
                 schedule: Optional[ScheduleConf],
                 ticket: TicketInfo,
                 limits: LimitsConf,
                 footer_defvars: Dict,
                 detector_name: str):

        self.name = name
        self.enabled = enabled
        self.schedule = schedule
        self.ticket = ticket
        self.limits = limits
        self.footer_defvars = footer_defvars
        self.detector_name = detector_name


class DetectorConf:
    def __init__(self,
                 identity: Identity,
                 name: str,
                 store: str,
                 rules: List[RuleConf],
                 im_engine: EventMeasImBuilderConf,
                 video_args: Optional[VideoConf] = None):

        self.identity = identity
        self.name = name
        self.store = store
        self.rules = rules
        self.im_engine = im_engine
        self.video = video_args
