from enum import IntEnum
from typing import Dict, Final, List, Optional

from edge.image.types.types import EventMeasImBuilderConf
from edge.support.mask import MaskConf
from edge.support.schedule import ScheduleConf
from edge.types.identity import Identity

from speedcontrol.detectors.types.conf import VideoConf
from speedcontrol.events.ticket import TicketInfo


class DirLimit(IntEnum):
    CODE_U = 1
    CODE_A = 11
    CODE_R = 12
    CODE_NOWAY = 10


class RuleLimits:
    def __init__(self, lanes: List[str], dir_limit: DirLimit):
        self.lanes = lanes
        self.dir: Final[DirLimit] = dir_limit


class RuleConf:
    def __init__(self,
                 name: str,
                 enabled: bool,
                 schedule: Optional[ScheduleConf],
                 ticket: TicketInfo,
                 limits: RuleLimits,
                 footer_defvars: Dict):

        self.name = name
        self.enabled = enabled
        self.schedule = schedule
        self.ticket = ticket
        self.limits = limits
        self.footer_defvars = footer_defvars


class DetectorConf:
    def __init__(self,
                 identity: Identity,
                 name: str,
                 store: str,
                 rules: List[RuleConf],
                 scene_mask: MaskConf,
                 imbuilder: EventMeasImBuilderConf,
                 video: Optional[VideoConf]):

        self.identity = identity
        self.name = name
        self.store = store
        self.rules = rules
        self.scene_mask = scene_mask
        self.imbuilder = imbuilder
        self.video = video
