from typing import Dict, List

from edge.services.types.tz import ITzService
from edge.types.origin import Origin
from speedcontrol.events.detector import DetectorInfo
from speedcontrol.events.evidence import BaseEvidence


class Evidence(BaseEvidence):
    def __init__(self,
                 origin: Origin,
                 ts: int,
                 detector_info: DetectorInfo,
                 evidence_seats_jnode: List[Dict]):

        super().__init__(BaseEvidence.EVENT_TYPE__SEATBELT, origin, ts)
        self.detector_info = detector_info
        self._evidence_seats_jnode = evidence_seats_jnode

    def serialize(self, tz_service: ITzService) -> Dict:
        jnode = BaseEvidence.serialize(self, tz_service)

        best_waypoints_jnode = None
        for waypoint in self._waypoints:
            if waypoint.name == waypoint.NAME_VIOLATION_BEST:
                best_waypoints_jnode = waypoint.serialize(tz_service)

        jnode['evidence'] = {
            'detector': {
                'name': self.detector_info.name,
                'ticket': {
                    'name': self.detector_info.rule.ticket.name
                }
            },
            'measurements': {
                'seats': self._evidence_seats_jnode,
                'waypoints': {
                    'best': best_waypoints_jnode
                }
            }
        }

        return jnode
