from typing import Dict

from edge.types.shape.quad import encode__quadi
from edge.types.shape.rect import encode__recti
from speedcontrol.events.vrp import VrpType, Vrp
from speedcontrol.trackset.wa import WaTrack


def encode__vrp_type(self: VrpType) -> Dict:
    jnode = {
        'country': {
            'numeric': self.country_num,
            'alpha-2': self.country_a2
        }
    }

    info_jnode: Dict[str, str] = dict()
    if self.attributes:
        info_jnode['attributes'] = self.attributes

    if self.template:
        info_jnode['template'] = self.template

    if info_jnode:
        jnode['info'] = [info_jnode]

    return jnode


def encode__vrp_extra_waga(self: WaTrack) -> Dict:
    jnode = {
        'provider': self.provider,
        'front-text': self.vrp_front,
        'back-text': self.vrp_back
    }

    if self.vrp_front_w:
        jnode['front-weight'] = self.vrp_front_w

    if self.vrp_back_w:
        jnode['back-weight'] = self.vrp_back_w

    return jnode


def encode__vrp(self: Vrp) -> Dict:
    jnode = {
        'weight': self.text_conf,
        'type': encode__vrp_type(self.type),
        'bbox-2': encode__recti(self.text_quad.bbox_recti),
        'bbox-4': encode__quadi(self.text_quad),
        'text': {
            'ucode': self.text_ucode
        }
    }

    if self.wa_track:
        jnode['extra'] = [
            encode__vrp_extra_waga(self.wa_track)
        ]

    return jnode
