from typing import Final, List, Optional

from edge.types.blob import MemBlob
from edge.types.origin import Origin

from speedcontrol.events.waypoint import Waypoint
from speedcontrol.events.waypoint.measurement import BestWaypoint


class Measurement:
    def __init__(self, uuid: str, origin: Optional[Origin], waypoints: List[Waypoint]):
        self.uuid: Final[str] = uuid
        self.origin: Final[Origin | None] = origin
        self.waypoints: Final[List[Waypoint]] = waypoints

    @property
    def best_waypoint(self) -> Optional[BestWaypoint]:
        waypoints = [x for x in self.waypoints if x.name == x.NAME_BEST]
        return waypoints[0] if len(waypoints) == 1 else None

    def get_blobs(self) -> List[MemBlob]:
        blobs: List[MemBlob] = []
        for waypoint in self.waypoints:
            blobs.extend(waypoint.images)

        return blobs
