from typing import Dict, Final, List, Optional

from edge.types.dir import Dir
from edge.types.shape.quad import QuadI
from edge.types.shape.rect import RectI
from speedcontrol.events.spotspeed import SpotSpeeds

from speedcontrol.events.vrp import Vrp
from speedcontrol.events.position import Position
from speedcontrol.trackset.wa import WaTrack


class ViolationVehicle:
    def __init__(self,
                 position: Optional[Position] = None,
                 direction: Optional[Dir] = None,
                 direction_alias: Optional[str] = None,
                 plate_rects: Optional[List[RectI]] = None,
                 plate_quads: Optional[List[QuadI]] = None):

        self.position: Final[Position | None] = position
        self.dir: Final[Dir | None] = direction
        self.dir_alias: Final[str | None] = direction_alias
        self.plate_rects: Final[List[RectI]] = plate_rects or list()
        self.plate_quads: Final[List[QuadI]] = plate_quads or list()


class Vehicle:
    def __init__(self,
                 plates: List[Vrp],
                 length: Optional[float],
                 direction: Dir,
                 direction_alias: Optional[str],
                 position: Optional[Position],
                 type_jnode: Optional[Dict],
                 seats_jnode: Optional[List[Dict]],
                 wa_track: Optional[WaTrack] = None,
                 spotspeeds: Optional[SpotSpeeds] = None):

        self.plates: Final[List[Vrp]] = plates
        self.length: Final[float | None] = length
        self.dir: Final[Dir] = direction
        self.dir_alias: Final[str | None] = direction_alias
        self.position: Final[Position | None] = position
        self.type_jnode: Final[Dict | None] = type_jnode
        self.seats_jnode: Final[List[Dict] | None] = seats_jnode

        self.spotspeeds: Final[SpotSpeeds] = spotspeeds or SpotSpeeds()
        self.wa_track: Final[WaTrack | None] = wa_track

    @property
    def mmr_class(self) -> str:
        return self.type_jnode and self.type_jnode.get('class') or 'UNKNOWN'

    @property
    def mmr_make(self) -> Optional[str]:
        if self.type_jnode:
            return self.type_jnode.get('make')

    @property
    def mmr_model(self) -> Optional[str]:
        if self.type_jnode:
            return self.type_jnode.get('model')

    @property
    def vrp_absent(self) -> bool:
        return len(self.plates) == 0

    @property
    def vrp_text_ucode(self) -> Optional[str]:
        for plate in self.plates:
            return plate.text_ucode

    @property
    def vrp_text_utf8(self) -> Optional[bytes]:
        for plate in self.plates:
            return plate.text_utf8

    @property
    def vrp_country_num(self) -> Optional[int]:
        for plate in self.plates:
            return plate.type.country_num

    @property
    def vrp_country_a2(self) -> Optional[str]:
        for plate in self.plates:
            return plate.type.country_a2
