from typing import Final, Optional

from edge.types.shape.quad import QuadI

from speedcontrol.trackset.wa import WaTrack


class VrpType:
    """
    The type of Vehicle Registration Plate
    """
    def __init__(self, country_a2: str, country_num: int, attributes: Optional[str], template: Optional[str]):
        self.country_a2: Final[str] = country_a2
        self.country_num: Final[int] = country_num
        self.attributes: Final[str | None] = attributes
        self.template: Final[str | None] = template


class Vrp:
    """
    Vehicle Registration Plate
    """
    def __init__(self,
                 vrp_type: VrpType,
                 text_quad: QuadI,
                 text_ucode: str,
                 text_conf: int,
                 wa_track: Optional[WaTrack] = None):

        self.type: Final[VrpType] = vrp_type
        self.text_quad: Final[QuadI] = text_quad
        self.text_ucode: Final[str] = text_ucode
        self.text_utf8: Final[bytes] = text_ucode.encode('utf-8')
        self.text_conf: Final[int] = text_conf

        self.wa_track: Final[WaTrack | None] = wa_track
