from typing import Dict, Final, List, Optional

from edge.services.types.tz import ITzService
from edge.types.blob import MemBlob
from edge.types.location import Location

from speedcontrol.events.jnode.location import encode__location


class Waypoint:
    NAME_CATCH = 'catch'
    NAME_BEST = 'best'
    NAME_LOST = 'lost'
    NAME_VIOLATION = 'violation'
    NAME_VIOLATION_BEST = 'violation-best'
    NAME_PRE_STOPBAR = 'pre-stopbar'
    NAME_POST_STOPBAR = 'post-stopbar'
    NAME_INIT = 'init'
    NAME_FINI = 'fini'

    def __init__(self,
                 name: str,
                 main_ts: int,
                 control_ts: Optional[int] = None,
                 location: Optional[Location] = None,
                 images: List[MemBlob] | None = None):

        self.name: Final[str] = name
        self.main_ts: Final[int] = main_ts
        self.control_ts: Final[int | None] = control_ts
        self.location: Final[Location] = location or Location()
        self.images: Final[List[MemBlob]] = images or []

    def serialize(self, tz_service: ITzService, with_images: bool = True) -> Dict:
        jnode = {
            'time': tz_service.map_ts_usec_to_isostr(self.main_ts)
        }

        if self.control_ts is not None:
            jnode['control-time'] = tz_service.map_ts_usec_to_isostr(self.control_ts)

        if self.location.provided:
            jnode['location'] = encode__location(self.location)

        if with_images:
            images_jnode: Dict[str, Dict] = dict()

            for image in self.images:
                images_jnode[image.profile] = image.serialize(tz_service)

            if len(images_jnode):
                jnode.update({'images': images_jnode})

        return jnode
