from typing import Dict, Final, List, Optional

from edge.services.types.tz import ITzService
from edge.types.blob import MemBlob
from edge.types.location import Location

from speedcontrol.events.jnode.location import encode__location
from speedcontrol.events.jnode.vehicle import encode__vehicle
from speedcontrol.events.vehicle import Vehicle
from speedcontrol.events.waypoint import Waypoint


class BestWaypoint(Waypoint):
    DIR_UNKNOWN = 'unknown'
    DIR_APPROACHING = 'approching'
    DIR_RECEDING = 'receding'

    def __init__(self,
                 main_ts: int,
                 control_ts: Optional[int],
                 location: Location,
                 images: List[MemBlob],
                 vehicle: Vehicle,
                 direction: str):

        super().__init__(Waypoint.NAME_BEST, main_ts, control_ts, location, images)

        self.vehicle: Final[Vehicle] = vehicle
        self.direction: Final[str] = direction

    def serialize(self, tz_service: ITzService, with_images: bool = True) -> Dict:
        jnode = Waypoint.serialize(self, tz_service, with_images)

        # replaces the 'vehicle' from Waypoint.serialize()
        jnode.update({
            'vehicle': encode__vehicle(self.vehicle)
        })

        if self.direction != self.DIR_UNKNOWN:
            jnode.update({
                'dir': self.direction
            })

        if self.location.provided:
            # TODO: replaces the 'images' from Waypoint.serialize()
            jnode.update({
                'location': encode__location(self.location)
            })

        images_jnode: Dict[str, Dict | List[Dict]] = dict()
        if with_images:
            images_jnode['license-plates'] = []
            for blob in self.images:
                blob_jnode = blob.serialize(tz_service)
                if blob.profile == 'plate':
                    images_jnode['license-plates'].append(blob_jnode)
                else:
                    images_jnode[blob.profile] = blob_jnode

        if len(images_jnode):
            # replaces the 'images' from Waypoint.serialize()
            jnode.update({
                'images': images_jnode
            })

        return jnode
