from typing import Dict, Final, List, Optional

from edge.services.types.tz import ITzService
from edge.types.blob import MemBlob
from edge.types.location import Location

from speedcontrol.events.jnode.vehicle import encode__violation_vehicle
from speedcontrol.events.vehicle import ViolationVehicle
from speedcontrol.events.waypoint import Waypoint


class RedLightViolationWaypoint(Waypoint):
    def __init__(self,
                 name: str,
                 main_ts: int,
                 location: Location,
                 images: List[MemBlob],
                 vehicle: Optional[ViolationVehicle],

                 time_green_amber: Optional[int],
                 time_red: Optional[int],
                 time_red_amber: Optional[int],
                 time_halt: Optional[int]):
        """
        :param name:
        :param main_ts:
        :param location:
        :param vehicle:
        :param time_green_amber:
            value in usec
        :param time_red:
            value in usec
        :param time_red_amber:
            value in usec
        """
        super().__init__(name, main_ts, None, location, images)

        self.vehicle: Final[ViolationVehicle] = vehicle

        self.time_green_amber: Final[int | None] = time_green_amber
        self.time_red: Final[int | None] = time_red
        self.time_red_amber: Final[int | None] = time_red_amber
        self.time_halt: Final[int | None] = time_halt

    def serialize(self, tz_service: ITzService, with_images: bool = True) -> Dict:
        jnode = Waypoint.serialize(self, tz_service, with_images)

        vehicle_jnode: Optional[Dict] = encode__violation_vehicle(self.vehicle)
        if vehicle_jnode is not None:
            jnode.update({
                'vehicle': vehicle_jnode,
            })

        if self.time_green_amber is not None:
            jnode.update({'green-amber-time': self.time_green_amber})

        if self.time_red is not None:
            jnode.update({'red-time': self.time_red})

        if self.time_red_amber is not None:
            jnode.update({'red-amber-time': self.time_red_amber})

        if self.time_halt is not None:
            jnode.update({'halt-time': self.time_halt})

        return jnode
