import os
from typing import Dict, Optional


def encode__fact(event_jnode: Dict,
                 args_blob_ref: str,
                 args_ua_blob_ref: Optional[str] = None,
                 args_ua_blob_sign: Optional[bool] = False) -> Dict:

    arch_files = []
    seatbelt_event = event_jnode['type'] == 'seatbelt'

    def add_arch_file(name: str, altref: Optional[str] = None) -> None:
        arch_file = {
            'ref': name,
            'name': name
        }

        if altref:
            arch_file['altref'] = altref

        arch_files.append(arch_file)

    def analyze_measurements() -> None:
        measurements = event_jnode['measurements']

        for measurement in measurements:
            waypoints = measurement['waypoints']
            best_images = waypoints['best'].get('images', {})
            plate_images = best_images.get('license-plates', [])

            # thumb.jpeg
            thumb_im_name = best_images.get('thumb', {}).get('name')
            if thumb_im_name is not None:
                add_arch_file(thumb_im_name, 'thumb.jpeg')

            # scene.jpeg
            scene_im_name = best_images.get('scene', {}).get('name')
            if scene_im_name is not None:
                add_arch_file(scene_im_name, 'scene.jpeg')

            # vehicle.jpeg
            vehicle_im_name = best_images.get('vehicle', {}).get('name')
            if vehicle_im_name is not None:
                add_arch_file(vehicle_im_name, 'vehicle.jpeg')

            # overview.jpeg
            overview_im_name = best_images.get('overview', {}).get('name', None)
            if overview_im_name is not None:
                add_arch_file(overview_im_name, 'overview.jpeg')

            # plate.jpeg
            list(map(lambda plate_im: add_arch_file(plate_im['name'], 'plate.jpeg'), plate_images))

    def analyze_evidence() -> None:
        e_jnode = event_jnode.get('evidence', {})
        em_jnode = e_jnode.get('measurements')
        if not em_jnode:
            return

        emw_jnodes = em_jnode.get('waypoints', {})
        for waypoint in emw_jnodes.values():
            if type(waypoint) != list:
                waypoint = [waypoint]

            for w_jnode in waypoint:
                wi_jnodes = w_jnode.get('images', {})
                if seatbelt_event:
                    plate_image = wi_jnodes.get('plate', {}).get('name')
                    add_arch_file(plate_image, 'plate.jpeg')

                    # thumb.jpeg
                    thumb_im_name = wi_jnodes.get('thumb', {}).get('name')
                    if thumb_im_name is not None:
                        add_arch_file(thumb_im_name, 'thumb.jpeg')

                    # scene.jpeg
                    scene_im_name = wi_jnodes.get('scene', {}).get('name')
                    if scene_im_name is not None:
                        add_arch_file(scene_im_name, 'scene.jpeg')

                    # vehicle.jpeg
                    vehicle_im_name = wi_jnodes.get('vehicle', {}).get('name')
                    if vehicle_im_name is not None:
                        add_arch_file(vehicle_im_name, 'vehicle.jpeg')

                else:
                    for wi_jnode in wi_jnodes.values():
                        add_arch_file(wi_jnode['name'])

        emv_jnodes = em_jnode.get('videos', [])
        for emv_jnode in emv_jnodes:
            add_arch_file(emv_jnode['name'])

    add_arch_file(args_blob_ref, 'args.json')

    if args_ua_blob_ref is not None:
        add_arch_file(args_ua_blob_ref, 'args-ua.json')

        if args_ua_blob_sign:
            sign_name = os.path.splitext(args_ua_blob_ref)[0] + '.sign'
            cert_name = os.path.splitext(args_ua_blob_ref)[0] + '.crt'
            add_arch_file(sign_name, 'args-ua.sign')
            add_arch_file(cert_name, 'args-ua.crt')

    analyze_measurements()
    analyze_evidence()

    return {
        'args': event_jnode,
        'dist': arch_files
    }
