from typing import Optional

from edge import eventlog
from edge.support import fs

from speedcontrol.services.types.plateinfo import PlateInfoServiceConf, IPlateInfoService


class _PlateInfoService(IPlateInfoService):
    def __init__(self, conf: PlateInfoServiceConf):
        IPlateInfoService.__init__(self)
        if conf.info_path:
            self._ref_to_info_map = fs.load_json3(conf.info_path)
        else:
            self._ref_to_info_map = dict()

    def get_country_code(self, template_ref: int) -> int:
        return (template_ref >> 20) & 0xFFF

    def get_plate_attrs(self, template_ref: int) -> Optional[str]:
        plate_info = self._ref_to_info_map.get(str(template_ref)) or dict()
        if not plate_info:
            eventlog.log_w(f'fail: unsupported (template-ref:{template_ref})')

        return plate_info.get('attrs')

    def get_plate_template(self, template_ref: int) -> Optional[str]:
        plate_info = self._ref_to_info_map.get(str(template_ref)) or dict()
        if not plate_info:
            eventlog.log_w(f'fail: unsupported (template-ref:{template_ref})')

        return plate_info.get('template')


def init_service(conf: PlateInfoServiceConf) -> IPlateInfoService:
    return _PlateInfoService(conf)
