from typing import Dict

from edge.support import fs

from speedcontrol.services.types.systest import SystestServiceConf, ISystestService


class _StubSystestService(ISystestService):
    def __init__(self):
        super().__init__()

    def serialize(self, jnode: Dict) -> Dict:
        return jnode


class _SystestService(ISystestService):
    def __init__(self, args: SystestServiceConf):
        super().__init__()

        jnode = fs.load_json3(args.conf_path, default=dict())
        rw_jnode = jnode.get('rw') or dict()
        self._crt_jnode = rw_jnode.get('certificate')

    def serialize(self, jnode: Dict) -> Dict:
        if self._crt_jnode:
            jnode.update({'certificate': self._crt_jnode})

        return jnode


def init_service(conf: SystestServiceConf) -> ISystestService:
    if conf.conf_path:
        systest = _SystestService(conf)
    else:
        systest = _StubSystestService()

    return systest
