from typing import Optional, Dict, Self

from edge import eventlog
from edge.types.exceptions import JnodeError
from speedcontrol.services.types.raas import RaasServiceConf


class JsonRaasServiceArgs(RaasServiceConf):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @classmethod
    def deserialize(cls, raas_jnode: Optional[Dict]) -> Optional[Self]:
        """
        :raise JnodeError:
        """
        if raas_jnode is None:
            return None

        # deserialize #/host
        raas_host = raas_jnode.get('host')
        if not (type(raas_host) in [str] and 1 <= len(raas_host) <= 16):
            eventlog.log_e_param('#/host')
            raise JnodeError

        # deserialize #/port
        raas_port = raas_jnode.get('port')
        if not (type(raas_port) in [str] and 1 <= len(raas_port) <= 16):
            eventlog.log_e_param('#/port')
            raise JnodeError

        # deserialize #/user
        raas_user = raas_jnode.get('user')
        if not (type(raas_user) in [str] and 1 <= len(raas_user) <= 16):
            eventlog.log_e_param('#/user')
            raise JnodeError

        # deserialize #/pass
        raas_pass = raas_jnode.get('pass')
        if not (type(raas_pass) in [str] and 1 <= len(raas_pass) <= 16):
            eventlog.log_e_param('#/pass')
            raise JnodeError

        return cls(raas_host, raas_port, raas_user, raas_pass)
