from typing import Dict, Self

from edge import eventlog
from edge.types.exceptions import JnodeError
from speedcontrol.services.types.sign import SignServiceConf


class JsonSignServiceArgs(SignServiceConf):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @classmethod
    def deserialize(cls, sign_jnode: Dict, log_level: str) -> Self:
        """
        :param sign_jnode:
            main.conf#/services/sign
        :param log_level:
            none | error | warn | info | debug | trace

        :rtype: JsonSignServiceArgs
        :raises JnodeError:
        """
        provider_jnode = sign_jnode.get('provider')
        if provider_jnode is None:
            eventlog.log_e_param('#/provider')
            raise JnodeError

        iit_conf = provider_jnode.get('iit')
        if type(iit_conf) not in [str]:
            eventlog.log_e_param('#/iit')
            raise JnodeError

        return cls(iit_conf, log_level)
