import os
from typing import Dict, Self

from edge import eventlog
from edge.types.exceptions import JnodeError
from speedcontrol.services.types.stats import StatsServiceConf


class JsonStatServiceArgs(StatsServiceConf):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @classmethod
    def deserialize(cls, monitoring_jnode: Dict) -> Self:
        """
        :raise JnodeError:
        """
        stat_filepath = monitoring_jnode.get('path')
        if not (type(stat_filepath) in [str] and len(stat_filepath) < 64 and os.path.isabs(stat_filepath)):
            eventlog.log_e_param('#/path')
            raise JnodeError

        stat_tout_sec = monitoring_jnode.get('tout')
        if not (int == type(stat_tout_sec) and 0 <= stat_tout_sec <= 255):
            eventlog.log_e_param('#/tout')
            raise JnodeError

        return cls(stat_filepath, stat_tout_sec)
