import datetime
import time

from typing import Dict

from edge import eventlog
from edge.types.exceptions import JnodeError

from speedcontrol.services.types.trainer import TrainerServiceConf


def decode__trainer_service_conf(jnode: Dict) -> TrainerServiceConf:
    """
    :raise JnodeError:
    """
    try:
        db_directory = jnode['directory']
        min_df = jnode.get('min-df', 0.1)
        time_min_max = jnode['limits'].get('time-min-max', [])
        lanes = jnode['limits'].get('lanes', [])

        min_time = datetime.time(hour=0, minute=0, second=0)
        max_time = datetime.time(hour=23, minute=59, second=59)
        if len(time_min_max) == 2:
            raw_min_time, raw_max_time = time_min_max
            min_struct_time = time.strptime(raw_min_time, "%H:%M")
            max_struct_time = time.strptime(raw_max_time, "%H:%M")

            min_time = datetime.time(min_struct_time.tm_hour, min_struct_time.tm_min)
            max_time = datetime.time(max_struct_time.tm_hour, max_struct_time.tm_min)

        lr = jnode['limits'].get('lr', TrainerServiceConf.LR_ANY)

        return TrainerServiceConf(db_directory, lanes, min_time, max_time, min_df, lr)
    except KeyError:
        eventlog.log_e('fail: plate_absent_db jnode have invalid keys')

    raise JnodeError
