from typing import Dict, List, Optional

from edge.types.track import LaneTrack, DirTrack
from speedcontrol.trackset.jnode.dir import decode__dir_track
from speedcontrol.trackset.jnode.lane import decode__lane_track
from speedcontrol.trackset.jnode.le import decode__le_track
from speedcontrol.trackset.jnode.lr import LrTrack, decode__lr_track
from speedcontrol.trackset.jnode.ra import RaTrack, decode__ra_track
from speedcontrol.trackset.jnode.wa import WaTrack, decode__wa_track
from speedcontrol.trackset.tr import TrTrack
from speedcontrol.trackset.trackset import Trackset


def decode__trackset(jnode: Dict) -> Optional[Trackset]:
    lr_track_jnode = jnode.get('Lr')
    lr_alarms_jnode = jnode.get('Alarms')
    lr_track: Optional[LrTrack] = lr_track_jnode and decode__lr_track(lr_track_jnode, lr_alarms_jnode)

    ra_track_jnode = jnode.get('Ra')
    ra_track: Optional[RaTrack] = ra_track_jnode and decode__ra_track(ra_track_jnode)

    wa_track_jnode = jnode.get('Wa')
    wa_track: Optional[WaTrack] = wa_track_jnode and decode__wa_track(wa_track_jnode)

    lane_jnode = jnode.get('Lane')
    lane_track: Optional[LaneTrack] = lane_jnode and decode__lane_track(lane_jnode)

    dir_jnode = jnode.get('Dir')
    dir_track: Optional[DirTrack] = dir_jnode and decode__dir_track(dir_jnode)

    if any((lr_track, ra_track, wa_track, dir_track, lane_track)):
        tr_track = TrTrack.create(lr_track, ra_track, dir_track, lane_track)
        return Trackset(jnode=jnode,
                        tr_track=tr_track,
                        lr_track=lr_track,
                        ra_track=ra_track,
                        wa_track=wa_track)


def decode__tracksets_le(jnode: Dict) -> List[Trackset]:
    le_tracks_jnode: List[Dict] | None = jnode.get('leddar')
    le_tracks = [decode__le_track(x) for x in le_tracks_jnode]
    return [Trackset(jnode=jnode, le_track=x) for x in le_tracks if x]
