from typing import Dict, Final, Optional
from uuid import uuid4

from edge.types.dir import Dir

from speedcontrol.trackset.le import LeTrack
from speedcontrol.trackset.lr import LrTrack
from speedcontrol.trackset.ra import RaTrack
from speedcontrol.trackset.tr import TrTrack
from speedcontrol.trackset.wa import WaTrack


class Trackset:
    """
    An immutable set of tracks that are related to the same vehicle.
    It should always be possible to create the BaseEvent from the Trackset
    """
    def __init__(self,
                 jnode: Optional[Dict] = None,
                 tr_track: Optional[TrTrack] = None,
                 lr_track: Optional[LrTrack] = None,
                 ra_track: Optional[RaTrack] = None,
                 le_track: Optional[LeTrack] = None,
                 wa_track: Optional[WaTrack] = None):

        self.uuid: Final[str] = str(uuid4())

        self.jnode: Final[Dict | None] = jnode
        self.tr_track: Final[TrTrack | None] = tr_track
        self.lr_track: Final[LrTrack | None] = lr_track
        self.ra_track: Final[RaTrack | None] = ra_track
        self.le_track: Final[LeTrack | None] = le_track
        self.wa_track: Final[WaTrack | None] = wa_track

    def get_direction(self) -> Dir:
        return Dir.CODE_U if self.tr_track is None else self.tr_track.dir
