import os
import sys
import ujson
import signal

from time import sleep
from typing import List
from twisted.internet import reactor

from edge import eventlog
from edge.support.argset import ArgSet

from tracker.kernel import Kernel
from tracker.types.kernel import decode__kernel_conf


# noinspection PyUnusedLocal
def handle_sighup(signal_nubmer, interrupted_stack_frame) -> None:
    # traceback.print_stack(interrupted_stack_frame)
    pass


# noinspection PyUnresolvedReferences,PyBroadException
def run_module(raw_arg_set: List[str]) -> None:
    arg_set = ArgSet(raw_arg_set)
    if arg_set.mode == ArgSet.MODE_NONE:
        print("The command line's arguments are invalid")
        sys.exit(1)

    if arg_set.mode == ArgSet.MODE_STARTUP:
        module_conf_name = 'tracker.{}'.format(arg_set.module_id)
        eventlog.open_log(module_conf_name, arg_set.log_level)
        eventlog.log_i('startup...')

        reactor.addSystemEventTrigger('after', 'startup', eventlog.log_i, 'startup... done')
        reactor.addSystemEventTrigger('before', 'shutdown', eventlog.log_i, 'shutdown...')
        signal.signal(signal.SIGHUP, handle_sighup)

        main_conf_abspath = os.path.join('/etc/edge', module_conf_name, 'main.conf')
        misc_conf_abspath = os.path.join('/etc/edge', module_conf_name, 'misc.conf')
        try:
            kernel_conf = None
            trottle_wait_args = False
            while kernel_conf is None:
                if trottle_wait_args:
                    sleep(15)

                trottle_wait_args = True

                # deserialize main.conf
                try:
                    with open(main_conf_abspath) as main_conf_jnode:
                        main_conf_jnode = ujson.load(main_conf_jnode)

                except Exception as x:
                    eventlog.log_e(f'failed to load {main_conf_abspath} ({x})')
                    continue

                # deserialize misc.conf
                try:
                    with open(misc_conf_abspath) as misc_conf:
                        misc_conf_jnode = ujson.load(misc_conf)

                except Exception as x:
                    eventlog.log_e(f'failed to load {misc_conf_abspath} ({x})')
                    continue

                # deserialize kernerl-args
                try:
                    kernel_conf = decode__kernel_conf(main_conf_jnode, misc_conf_jnode)

                except Exception as x:
                    eventlog.log_e(f'failed to read args ({x})')
                    continue

            kernel = Kernel(reactor, kernel_conf)
            kernel.run()

        finally:
            eventlog.log_i('shutdown... done')
            eventlog.close_log()


if __name__ == '__main__':
    run_module(sys.argv)
