import os
from errno import ENOENT

from edge import eventlog
from edge.support import fs
from edge.types.imageref import ImageRef
from tracker.services.stats import SceneEventCounters
from tracker.services.types.scene_emitter import ISceneEmitter, SceneEmitterConf
from tracker.types.jnode.lrra import encode__lrra_scene
from tracker.types.lrra import LrRaScene


class SceneEmitter(ISceneEmitter):
    def __init__(self, ident: int, conf: SceneEmitterConf, stats: SceneEventCounters):
        ISceneEmitter.__init__(self)
        eventlog.log_d(f'ident:{ident} dirs:{repr(conf.event_dirs)}')

        self._stat_counters = stats

        self._ident = ident
        self._event_dirs = conf.event_dirs
        self._skip_rate = conf.skip_rate
        self._frame_counter = 0

        list(map(fs.silent_mkdir_r, self._event_dirs))

    def consume_scene(self, im_ref: ImageRef, tr_scene: LrRaScene) -> None:
        self._frame_counter += 1
        self._frame_counter %= self._skip_rate

        if self._frame_counter:
            return

        # get the event's filename.
        event_filename = str(im_ref.ts)

        # send the event to target directories.
        # noinspection PyBroadException
        try:
            event_args = encode__lrra_scene(tr_scene, im_ref)
            for event_dir in self._event_dirs:
                try_count = 2
                scene_dir_present = True
                event_abspath = os.path.join(event_dir, event_filename)
                while try_count > 0 and scene_dir_present:
                    try:
                        with open(event_abspath, 'w') as event_file:
                            event_file.write(event_args)

                        self._stat_counters.inc_scene_count(self._ident)
                        break

                    except IOError as x:
                        if x.errno == ENOENT:
                            scene_dir_present = False

                    if not scene_dir_present:
                        fs.silent_mkdir_r(event_dir)
                        scene_dir_present = True

                    try_count -= 1

        except Exception:
            eventlog.log_x()
