from typing import Dict

from edge import eventlog
from edge.services.types.jnode.lane import decode__lanes_conf
from edge.services.types.lane import LaneServiceConf


def decode__road_conf(jnode: Dict) -> LaneServiceConf:
    """
    :param jnode:
        /etc/edge/tracker.10/road.conf#/
    :raise ValueError:
    """
    if not len(jnode):
        eventlog.log_i('using stub LaneDetector')
        stub_lane_service = LaneServiceConf([], 0, 0)
        return stub_lane_service

    # decode #/road
    road_jnode = jnode.get('road')
    if not type(road_jnode) in [dict]:
        eventlog.log_e_param('#/road')
        raise ValueError()

    # decode #/scene-wh
    wh_jnode = jnode.get('scene-wh')
    if not wh_jnode:
        eventlog.log_e_param('#/scene-wh')
        raise ValueError()
    elif len(wh_jnode) != 2:
        eventlog.log_e_param('#/scene-wh')
        raise ValueError()

    mask_w, mask_h = wh_jnode
    if not (type(mask_w) in [int] and 0 <= mask_w <= 65535):
        eventlog.log_e_param('#/mask/scene-wh/0')
        raise ValueError()

    elif not (type(mask_h) in [int] and 0 <= mask_h <= 65535):
        eventlog.log_e_param('#/mask/scene-wh/1')
        raise ValueError()

    # decode #/road/lanes
    lanes_jnode = road_jnode.get('lanes', [])
    lane_service_conf = decode__lanes_conf(lanes_jnode, mask_w, mask_h)

    return lane_service_conf
