from os.path import isabs
from typing import Dict, Self, Final

from edge.services.types.service import IService
from tracker.types.lrra import LrRaTrackset


class ImageQueueConf:
    def __init__(self, endpoint: str, min_file_count: int, max_file_count: int):
        self.endpoint = endpoint
        self.min_file_count = min_file_count
        self.max_file_count = max_file_count
        self.log_level = 'info'

    @classmethod
    def deserialize(cls, store_jnode: Dict) -> Self:
        endpoint = store_jnode.get('endpoint')
        if not (str == type(endpoint) and len(endpoint) < 64 and isabs(endpoint)):
            raise ValueError('main.conf#/store/endpoint')

        max_file_count = store_jnode.get('max-file-count', 15)
        if not (int == type(max_file_count) and 15 <= max_file_count <= 50):
            raise ValueError('main.conf#/store/max-file-count')

        return ImageQueueConf(endpoint, max_file_count - 5, max_file_count)


class TrackDebouncerConf:
    def __init__(self, debounce_tout: int, imq: ImageQueueConf) -> None:
        self.debounce_tout: Final[int] = debounce_tout
        self.imq: Final[ImageQueueConf] = imq


class ITrackDebouncer(IService):
    def consume_track(self, lrra_track: LrRaTrackset) -> None:
        raise NotImplementedError()
