import ujson

from typing import Any, Dict, Optional

from edge.types.imageref import ImageRef
from edge.types.jnode.imageref import encode__image_ref
from edge.types.jnode.track import encode__lane_track_item, encode__dir_track_item
from edge.types.track import LaneTrack, DirTrack

from tracker.types.jnode.lr import encode__lr_scene_item
from tracker.types.jnode.ra import encode_ra_track, encode__ra_scene_item
from tracker.types.lr import LrTrackSet
from tracker.types.lrra import LrRaScene
from tracker.types.ra import RaTrack


def encode__lrra_scene(self: LrRaScene, imref: ImageRef) -> str:
    jnode = {
        'Image': encode__image_ref(imref),
        'LrRa': [
            {
                'Lr': encode__lr_scene_item(x.lr_item),
                'Ra': encode__ra_scene_item(x.ra_item, x.ra_bbox)
            }
            for x in self.tr_items
        ],
        'Lr': [
            encode__lr_scene_item(x) for x in self.lr_items
        ],
        'Ra': [
            encode__ra_scene_item(x.ra_item, x.ra_bbox) for x in self.ra_items
        ]
    }

    #
    # https://pypi.python.org/pypi/ujson
    #
    # ensure_ascii limits output to ASCII and escapes all extended characters above 127. Default is true.
    # If your end format supports UTF-8 setting this option to false is highly recommended to save space.
    return ujson.dumps(jnode, ensure_ascii=False)


def encode__lr_trackset(self: LrTrackSet, lr_imref: Optional[ImageRef]) -> Dict[str, Any]:
    jnode = self.lr_jnode

    if lr_imref and lr_imref.url:
        jnode['Image']['Url'] = lr_imref.url

    return jnode


def encode__trackset(lr_imref: Optional[ImageRef],
                     lr_trackset: Optional[LrTrackSet],
                     ra_track: Optional[RaTrack],
                     lane_track: Optional[LaneTrack],
                     dir_track: Optional[DirTrack]) -> str:
    jnode = dict()

    if lr_trackset is not None:
        jnode['Lr'] = encode__lr_trackset(lr_trackset, lr_imref)

        if lr_trackset.alarms_jnode:
            jnode['Alarms'] = lr_trackset.alarms_jnode

    waga_track_jnode: Optional[Dict] = lr_trackset and lr_trackset.wa_jnode
    if waga_track_jnode:
        jnode['Wa'] = waga_track_jnode

    if ra_track is not None:
        jnode['Ra'] = encode_ra_track(ra_track)

    if lane_track:
        jnode['Lane'] = {
            'Lanes': list(lane_track.lanes),
            'Trace': [encode__lane_track_item(x) for x in lane_track]
        }

    if dir_track:
        jnode['Dir'] = {
            'D': dir_track.dir.value,
            'Trace': [encode__dir_track_item(x) for x in dir_track]
        }

    trackset_utf8 = ujson.dumps(jnode, ensure_ascii=False)
    return trackset_utf8
