from typing import Final, List, Optional, Self

from edge.types.shape.rect import RectI
from tracker.types.lr import LrSceneItem, LrTrackSet
from tracker.types.ra import RaSceneItem, RaTrack


class LrRaSceneItem:
    def __init__(self,
                 lr_item: Optional[LrSceneItem],
                 ra_item: RaSceneItem,
                 match_rect: RectI):

        if not lr_item and not ra_item:
            raise ValueError

        self.lr_item: Final[LrSceneItem | None] = lr_item
        self.ra_item: Final[RaSceneItem] = ra_item
        self.ra_bbox: Final[RectI] = match_rect


class LrRaScene:
    def __init__(self,
                 tr_items: List[LrRaSceneItem],
                 lr_items: List[LrSceneItem],
                 ra_items: List[LrRaSceneItem]) -> None:
        """
        :param tr_items:
            The list of matched lr_scene_item and ra_scene_item
        :param lr_items:
            The list of not matched lr_scene_items
        :param ra_items:
            The list of not matched ra_scene_items
        """
        self.tr_items: Final[List[LrRaSceneItem]] = tr_items
        self.lr_items: Final[List[LrSceneItem]] = lr_items
        self.ra_items: Final[List[LrRaSceneItem]] = ra_items

    @classmethod
    def create(cls, tr_items: List[LrRaSceneItem], lr_items: List[LrSceneItem], ra_items: List[LrRaSceneItem]):
        return cls(tr_items, lr_items, ra_items)

    @classmethod
    def create_lr(cls, lr_items: List[LrSceneItem]) -> Self:
        return cls([], lr_items, [])

    @classmethod
    def create_ra(cls, ra_items: List[LrRaSceneItem]) -> Self:
        return cls([], [], ra_items)

    @classmethod
    def create_empty(cls) -> Self:
        return cls([], [], [])


class LrRaTrackset:
    def __init__(self, lr_trackset: Optional[LrTrackSet], ra_track: Optional[RaTrack]):
        if not lr_trackset and not ra_track:
            raise ValueError

        self.lr_trackset: Final[LrTrackSet | None] = lr_trackset
        self.ra_track: Final[RaTrack | None] = ra_track
